<?php

namespace App\Exceptions;

use App;
use Exception;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Foundation\Exceptions\Handler as ExceptionHandler;
use Illuminate\Session\TokenMismatchException;
use Redirect;
use Request;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class Handler extends ExceptionHandler {
  /**
   * A list of the exception types that should not be reported.
   *
   * @var array
   */
  protected $dontReport = [
    \Illuminate\Auth\AuthenticationException::class,
    \Illuminate\Auth\Access\AuthorizationException::class,
    \Symfony\Component\HttpKernel\Exception\HttpException::class,
    \Illuminate\Database\Eloquent\ModelNotFoundException::class,
    \Illuminate\Session\TokenMismatchException::class,
    \Illuminate\Validation\ValidationException::class,
  ];

  /**
   * Report or log an exception.
   *
   * This is a great spot to send exceptions to Sentry, Bugsnag, etc.
   *
   * @param  \Exception $exception
   * @return void
   */
  public function report( Exception $exception ) {
    if ( App::environment() == 'production' ) {
      if ( app()->bound( 'sentry' ) && $this->shouldReport( $exception ) ) {
        $sentry = app( 'sentry' );
        // Add tags context
        $session = $_REQUEST;
        $sentry->user_context( $session );
        $sentry->tags_context( [
          'referer'            => isset( $_SERVER[ 'HTTP_REFERER' ] ) ? $_SERVER[ 'HTTP_REFERER' ] : null,
          'referer-ip'         => Request::getClientIp(),
          'cadaris_user_id'    => \Auth::user()->id ?? 'sem id',
          'cadaris_user_email' => \Auth::user()->email ?? 'sem email',
        ] );
        app( 'sentry' )->captureException( $exception );
      }
    }
    parent::report( $exception );
  }

  /**
   * Render an exception into an HTTP response.
   *
   * @param  \Illuminate\Http\Request $request
   * @param  \Exception $exception
   * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response|\Symfony\Component\HttpFoundation\Response
   */
  public function render( $request, Exception $exception ) {
    // Se nao eh ajax e esta em debug, entao redireciona
    if ( !$request->ajax() && !config( 'app.debug', false ) ) {
      // Se caso eh uma rota invalida
      if ( $exception instanceof NotFoundHttpException ) {
        return Redirect::route( 'home' );
      }
      // Se caso eh um abort()
      if ( $exception instanceof HttpException ) {
        return Redirect::route( 'home' );
      }
      // Se caso o model nao existe no BD
      if ( $exception instanceof ModelNotFoundException ) {
        return Redirect::back(); // TODO: talvez back nao eh a melhor escolha
      }
    }
    if ( $exception instanceof TokenMismatchException ) {
      return Redirect::route( 'login' )->with( 'message', 'Sua Sessão Expirou!' );
    }
    // Senao exibe o erro por completo
    return parent::render( $request, $exception );
  }

  /**
   * Convert an authentication exception into an unauthenticated response.
   *
   * @param  \Illuminate\Http\Request $request
   * @param  \Illuminate\Auth\AuthenticationException $exception
   * @return \Illuminate\Http\Response
   */
  protected function unauthenticated( $request, AuthenticationException $exception ) {
    if ( $request->expectsJson() ) {
      return response()->json( [ 'error' => 'Unauthenticated.' ], 401 );
    }

    return redirect()->guest( 'login' );
  }
}
