<?php

namespace App\Http\Controllers\Cadastros;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Http\Requests\ClientesRequest;
use App\Models\Cadastros\ClientesCentrosDeCustos;
use App\Models\Cadastros\ClientesCentrosDeCustosContatos;
use App\Models\Cadastros\ClientesEmpresas;
use App\Models\Cadastros\ClientesEmpresasUnidades;
use Maatwebsite\Excel\Facades\Excel;
use PHPExcel_Style_Border;

class ClientesController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Cadastro_read ) )->only( [ 'index', 'show' ] );
    $this->middleware( CRP::role( CRP::P_Cadastro_edit ) )->only( [ 'create', 'store', 'edit', 'update', 'destroy', 'export_contatos_clientes_XLSX' ] );
  }

  /**
   * Exibe o formulario para inserir novo Cliente.
   *
   * @return \Illuminate\Http\JsonResponse
   */
  public function index() {
    $clientes = [];

    foreach ( ClientesEmpresas::orderBy( 'codigo' )->get() as $empresa ) {
      /** @var ClientesEmpresas $empresa */
      $emp           = [];
      $empID         = $empresa->id;
      $empCod        = $empresa->codigo;
      $emp[ 'text' ] = sprintf( '%02d - %s', $empCod, $empresa->nome );
      $emp[ 'href' ] = route( 'clientes.show', [ $empID, 0, 0 ] );
      $centros       = $empresa->CentrosDeCustos()->orderBy( 'codigo' )->get();
      $emp[ 'tags' ] = [ count( $centros ) ];

      foreach ( $centros as $centro ) {
        $cdc           = [];
        $depID         = $centro->id;
        $cdcCod        = $centro->codigo;
        $cdc[ 'text' ] = sprintf( '%02d.%02d - %s', $empCod, $cdcCod, $centro->nome );
        $cdc[ 'href' ] = route( 'clientes.show', [ $empID, $depID, 0 ] );
        $contas        = $centro->Contas()->orderBy( 'codigo' )->get();
        $cdc[ 'tags' ] = [ count( $contas ) ];

        foreach ( $contas as $conta ) {
          $cnt           = [];
          $cntID         = $conta->id;
          $cntCod        = $conta->codigo;
          $cnt[ 'text' ] = sprintf( '%02d.%02d.%02d - %s', $empCod, $cdcCod, $cntCod, $conta->nome );
          $cnt[ 'href' ] = route( 'clientes.show', [ $empID, $depID, $cntID ] );
          $cnt[ 'tags' ] = [];

          $cdc[ 'nodes' ][] = $cnt;
        }
        $emp[ 'nodes' ][] = $cdc;
      }
      $clientes[] = $emp;
    }

    $clientesJSON = json_encode( $clientes );
    return view( 'Cadastros.Clientes.index', compact( 'clientesJSON' ) );
  }

  /**
   * Exibe o formulario para inserir novo Cliente.
   *
   * @return \Illuminate\Http\Response
   */
  public function create() {

    return view( 'Cadastros.Clientes.create' );
  }

  /**
   * Armazena um novo Cliente no BD.
   *
   * @param ClientesRequest $request
   * @return \Illuminate\Http\Response
   */
  public function store( ClientesRequest $request ) {

    $emp_id = $request->get( 'empresa', '' );
    $cen_id = $request->get( 'centro', '' );

    if ( $emp_id == '' ) { // nova Empresa
      ClientesEmpresas::create( [
        'codigo' => $request->get( 'codigo' ),
        'nome'   => $request->getOrNull( 'nome' ),
        'obs'    => $request->getOrNull( 'obs' ),
      ] );

    } elseif ( $cen_id == '' ) { // novo Centro
      $dados = $request->all();

      /** @var ClientesEmpresas $emp */
      $emp = ClientesEmpresas::findOrFail( $emp_id );
      /** @var ClientesCentrosDeCustos $cdc */
      $cdc = $emp->CentrosDeCustos()->create( $request->all() );

      // Contatos
      $contato_nome  = array_get( $dados, 'contato_nome', [] );
      $contato_tel   = array_get( $dados, 'contato_tel', [] );
      $contato_cel   = array_get( $dados, 'contato_cel', [] );
      $contato_email = array_get( $dados, 'contato_email', [] );

      // insere os contatos do centro de custo
      for ( $i = 0; $i < count( $contato_nome ); $i++ ) {
        // cria novo contato
        $cdc->Contatos()->create( [
          'nome'  => $contato_nome[ $i ],
          'tel'   => $contato_tel[ $i ],
          'cel'   => $contato_cel[ $i ],
          'email' => $contato_email[ $i ],
        ] );
      }
    } else { // nova Conta
      /** @var ClientesEmpresas $emp */
      $emp = ClientesEmpresas::findOrFail( $emp_id );
      /** @var ClientesCentrosDeCustos $cdc */
      $cdc = $emp->CentrosDeCustos()->find( $cen_id );
      $cdc->Contas()->create( $request->all() );

    }

    flash( 'Cliente criado com sucesso!', 'success' );
    return redirect()->route( 'clientes.index' );
  }

  /**
   * Exibe os atributos do Cliente.
   *
   * @param $empCod
   * @param $cdcCod
   * @param $cntCod
   * @return \Illuminate\Http\Response
   */
  public function show( $empCod, $cdcCod, $cntCod ) {
    $empresa  = null;
    $centro   = null;
    $conta    = null;
    $unidades = null;
    $contatos = null;

    if ( $empCod <= 0 ) {
      return abort( '400', 'Falta parametro Codigo da Empresa' );
    } else {
      /** @var ClientesEmpresas $empresa */
      $empresa = ClientesEmpresas::findOrFail( $empCod );

      if ( $cdcCod <= 0 ) {
        // show Empresa
        $unidades = $empresa->Unidades;
        $tipo     = 1;

      } else {
        /** @var ClientesCentrosDeCustos $centro */
        $centro = $empresa->CentrosDeCustos()->findOrFail( $cdcCod );

        if ( $cntCod <= 0 ) {
          // show Centro
          $contatos = $centro->Contatos;
          $tipo     = 2;

        } else {
          // show Conta
          $conta = $centro->Contas()->findOrFail( $cntCod );
          $tipo  = 3;

        }
      }
    }


    return view( 'Cadastros.Clientes.show', compact( 'empresa', 'centro', 'conta', 'unidades', 'contatos', 'tipo', 'empCod', 'cdcCod', 'cntCod' ) );
  }

  /**
   * Exibe o formulario para editar um Cliente.
   *
   * @param $empCod
   * @param $cdcCod
   * @param $cntCod
   * @return \Illuminate\Http\Response
   */
  public function edit( $empCod, $cdcCod, $cntCod ) {
    $empresa  = null;
    $centro   = null;
    $conta    = null;
    $unidades = null;
    $contatos = null;

    if ( $empCod <= 0 ) {
      return abort( '400', 'Falta parametro Codigo da Empresa' );
    } else {
      /** @var ClientesEmpresas $empresa */
      $empresa = ClientesEmpresas::findOrFail( $empCod );

      if ( $cdcCod <= 0 ) {
        // show Empresa
        $unidades = $empresa->Unidades;
        $tipo     = 1;

      } else {
        /** @var ClientesCentrosDeCustos $centro */
        $centro = $empresa->CentrosDeCustos()->findOrFail( $cdcCod );

        if ( $cntCod <= 0 ) {
          // show Centro
          $contatos = $centro->Contatos;
          $tipo     = 2;

        } else {
          // show Conta
          $conta = $centro->Contas()->findOrFail( $cntCod );
          $tipo  = 3;

        }
      }
    }

    return view( 'Cadastros.Clientes.edit', compact( 'empresa', 'centro', 'conta', 'tipo', 'empCod', 'cdcCod', 'cntCod', 'unidades', 'contatos' ) );
  }

  /**
   * Atualiza os dados de um Cliente.
   *
   * @param ClientesRequest $request
   * @param $empCod
   * @param $cdcCod
   * @param $cntCod
   * @return \Illuminate\Http\Response
   */
  public function update( ClientesRequest $request, $empCod, $cdcCod, $cntCod ) {
    $dados = $request->all();
    if ( $empCod > 0 && $cdcCod == 0 && $cntCod == 0 ) {
      // update Empresa
      /** @var ClientesEmpresas $empresa */
      $empresa = ClientesEmpresas::findOrFail( $empCod );
      $empresa->update( [
        'codigo' => $request->get( 'codigo' ),
        'nome'   => $request->get( 'nome' ),
        'obs'    => $request->getOrNull( 'obs' ),
      ] );

    } elseif ( $empCod > 0 && $cdcCod > 0 && $cntCod == 0 ) {
      // update Departamento
      /** @var ClientesEmpresas $empresa */
      $empresa = ClientesEmpresas::findOrFail( $empCod );
      /** @var ClientesCentrosDeCustos $centro */
      $centro = $empresa->CentrosDeCustos()->findOrFail( $cdcCod );
      $centro->update( $dados );
      $contatos = $centro->Contatos();

      // update contatos
      $contato_id    = array_get( $dados, 'contato_id', [] );
      $contato_nome  = array_get( $dados, 'contato_nome', [] );
      $contato_tel   = array_get( $dados, 'contato_tel', [] );
      $contato_cel   = array_get( $dados, 'contato_cel', [] );
      $contato_email = array_get( $dados, 'contato_email', [] );

      // funcao usada no array_map para selecionar o 'id', e tornar o array associativo em somente array
      $soID = function ( $value ) { return $value[ 'id' ]; };

      // analisa as diferencas
      // flat a lista de ids, ex: [1, 2, 3, 4]
      $contato_antigo = array_map( $soID, $contatos->get( [ 'id' ] )->toArray() );
      // calcula a diferença ( encontra os valores nao presente no $contato_id, mas que tinha no antigo) e reinicia o array associativo
      $contato_deletados = array_values( array_diff( $contato_antigo, $contato_id ) );

      // deleta os anteriores do BD
      ClientesCentrosDeCustosContatos::destroy( $contato_deletados );

      // insere os novos ou atualiza os editados
      for ( $i = 0; $i < count( $contato_nome ); $i++ ) {
        $cont = [
          'nome'  => $contato_nome[ $i ],
          'tel'   => $contato_tel[ $i ],
          'cel'   => $contato_cel[ $i ],
          'email' => $contato_email[ $i ],
        ];
        if ( $contato_id[ $i ] == '-1' ) { // cria novo endereco
          $centro->Contatos()->create( $cont );
        } else { // atualiza endereco com aquele id
          $centro->Contatos()->find( $contato_id[ $i ] )->update( $cont );
        }
      }

    } elseif ( $empCod > 0 && $cdcCod > 0 && $cntCod > 0 ) {
      // update Conta
      /** @var ClientesEmpresas $empresa */
      $empresa = ClientesEmpresas::findOrFail( $empCod );
      /** @var ClientesCentrosDeCustos $centro */
      $centro = $empresa->CentrosDeCustos()->findOrFail( $cdcCod );
      $conta  = $centro->Contas()->findOrFail( $cntCod );
      $conta->update( $dados );

    }

    flash( 'Cliente editado com sucesso!', 'success' );
    return redirect()->route( 'clientes.show', [ $empCod, $cdcCod, $cntCod ] );
  }

  /**
   * Remove um Cliente do BD.
   *
   * @param $empCod
   * @param $cdcCod
   * @param $cntCod
   * @return \Illuminate\Http\Response
   */
  public function destroy( $empCod, $cdcCod, $cntCod ) {

    if ( $empCod > 0 && $cdcCod == 0 && $cntCod == 0 ) {
      // delete Empresa
      $empresa = ClientesEmpresas::findOrFail( $empCod );
      $empresa->delete();

    } elseif ( $empCod > 0 && $cdcCod > 0 && $cntCod == 0 ) {
      // delete Departamento
      /** @var ClientesEmpresas $empresa */
      $empresa = ClientesEmpresas::findOrFail( $empCod );
      $centro  = $empresa->CentrosDeCustos()->findOrFail( $cdcCod );
      $centro->delete();

    } elseif ( $empCod > 0 && $cdcCod > 0 && $cntCod > 0 ) {
      // delete Conta
      /** @var ClientesEmpresas $empresa */
      $empresa = ClientesEmpresas::findOrFail( $empCod );
      /** @var ClientesCentrosDeCustos $centro */
      $centro = $empresa->CentrosDeCustos()->findOrFail( $cdcCod );
      $conta  = $centro->Contas()->findOrFail( $cntCod );
      $conta->delete();

    }

    flash( 'Cliente deletado com sucesso!', 'success' );
    return redirect()->route( 'clientes.index' );
  }

  /**
   * Exibe o formulario para inserir nova Unidade do Cliente.
   *
   * @param $empID
   * @return \Illuminate\Http\Response
   */
  public function unidades_create( $empID ) {
    $empresa = ClientesEmpresas::findOrFail( $empID );

    return view( 'Cadastros.Clientes.unidades_create', compact( 'empresa' ) );
  }

  /**
   * Armazena um nova Unidade do Cliente no BD.
   *
   * @param ClientesRequest $request
   * @param $empCod
   * @return \Illuminate\Http\Response
   */
  public function unidades_store( ClientesRequest $request, $empCod ) {
    /** @var ClientesEmpresas $empresa */
    $empresa = ClientesEmpresas::findOrFail( $empCod );
    $empresa->Unidades()->create( $request->all() );

    flash( 'Unidade do Cliente criado com sucesso!', 'success' );
    return redirect()->route( 'clientes.show', [ $empresa->id, 0, 0 ] );
  }

  /**
   * Exibe os atributos da Unidade do Cliente.
   *
   * @param $empID
   * @param $uniID
   * @return \Illuminate\Http\Response
   */
  public function unidades_show( $empID, $uniID ) {
    /** @var ClientesEmpresas $empresa */
    $empresa = ClientesEmpresas::findOrFail( $empID );
    $unidade = $empresa->Unidades()->find( $uniID );

    return view( 'Cadastros.Clientes.unidades_show', compact( 'empresa', 'unidade' ) );
  }

  /**
   * Exibe o formulario para editar uma Unidade do Cliente.
   *
   * @param $empID
   * @param $uniID
   * @return \Illuminate\Http\Response
   */
  public function unidades_edit( $empID, $uniID ) {
    /** @var ClientesEmpresas $empresa */
    $empresa = ClientesEmpresas::findOrFail( $empID );
    $unidade = $empresa->Unidades()->find( $uniID );

    return view( 'Cadastros.Clientes.unidades_edit', compact( 'empresa', 'unidade' ) );
  }

  /**
   * Atualiza os dados de uma Unidade do Cliente.
   *
   * @param ClientesRequest $request
   * @param $empCod
   * @param $uniID
   * @return \Illuminate\Http\Response
   */
  public function unidades_update( ClientesRequest $request, $empCod, $uniID ) {
    /** @var ClientesEmpresas $empresa */
    $empresa = ClientesEmpresas::findOrFail( $empCod );
    $unidade = $empresa->Unidades()->find( $uniID );
    $unidade->update( $request->all() );

    flash( 'Unidade do Cliente editado com sucesso!', 'success' );
    return redirect()->route( 'clientes.show', [ $empresa->id, 0, 0 ] );
  }

  /**
   * Remove uma Unidade do Cliente do BD.
   *
   * @param $empCod
   * @param $uniID
   * @return \Illuminate\Http\Response
   */
  public function unidades_destroy( $empCod, $uniID ) {
    /** @var ClientesEmpresas $empresa */
    $empresa = ClientesEmpresas::findOrFail( $empCod );
    $unidade = $empresa->Unidades()->find( $uniID );
    $unidade->delete();

    flash( 'Unidade do Cliente deletado com sucesso!', 'success' );
    return redirect()->route( 'clientes.show', [ $empresa->id, 0, 0 ] );
  }

  /**
   * Gera o XLSX dos Contatos dos Clientes.
   *
   * @return void
   */
  public function export_contatos_clientes_XLSX() {
    /** @var ClientesEmpresas[] $clientesEmpresas */
    $clientesEmpresas = ClientesEmpresas::orderBy( 'codigo' )->get();

    $XLSX   = [];
    $XLSX[] = [
      'NOME',
      'EMPRESA',
      'CENTRO DE CUSTO',
      'EMAIL',
      'TELEFONE',
      'CELULAR',
      'ENDEREÇO',
    ];
    foreach ( $clientesEmpresas as $clienteEmpresa ) {
      $empresaNome = $clienteEmpresa->getCodigoNome();
      $empresaEnd  = '';

      /** @var ClientesEmpresasUnidades $unidd */
      $unidd = $clienteEmpresa->Unidades()->first();
      if ( $unidd ) {
        $empresaEnd = $unidd->_EnderecoCompleto();
      }

      foreach ( $clienteEmpresa->CentrosDeCustos as $centrosDeCustos ) {
        $centroDeCustoNome = $centrosDeCustos->getCodigoNome();
        foreach ( $centrosDeCustos->Contatos as $contato ) {
          $XLSX[] = [
            $contato->nome,
            $empresaNome,
            $centroDeCustoNome,
            $contato->email,
            $contato->tel,
            $contato->cel,
            $empresaEnd,
          ];
        }
      }
    }

    Excel::create( 'Sistema Cadaris - Contatos dos Clientes', function ( $excel ) use ( $XLSX ) {
      /** @var \Maatwebsite\Excel\Writers\LaravelExcelWriter $excel */
      $excel->setTitle( 'Sistema Cadaris - Contatos dos Clientes' );
      $excel->setDescription( 'Sistema Cadaris - Contatos dos Clientes' );
      $excel->sheet( 'Contatos dos Clientes', function ( $sheet ) use ( $XLSX ) {
        /** @var \Maatwebsite\Excel\Classes\LaravelExcelWorksheet $sheet */
        // Dados
        $sheet->fromArray( $XLSX, null, 'A1', false, false );

        // Style geral
        $sheet->setStyle( [
          'font' => [
            'name' => 'Arial',
            'size' => 10
          ]
        ] );
        $sheet->setAutoSize( true );
        $sheet->setBorder( 'A1:G' . count( $XLSX ), PHPExcel_Style_Border::BORDER_THIN );

        // Style do Header
        $sheet->setHeight( 1, 25 );
        $sheet->cells( 'A1:G1', function ( $cells ) {
          /** @var \Maatwebsite\Excel\Writers\CellWriter $cells */
          $cells->setFontColor( '#ffffff' );
          $cells->setBackground( '#404040' );
          $cells->setAlignment( 'center' );
          $cells->setValignment( 'center' );
          $cells->setFontWeight( 'bold' );
        } );

        // Auto Filter
        $sheet->setAutoFilter();
      } );
    } )->download( 'xlsx' );
  }
}
