<?php

namespace App\Http\Controllers\Cadastros;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Models\Cadastros\ControleDePontos;
use App\Models\Cadastros\Funcionarios;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Response;

class ControleDePontoApiController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   */
  public function __construct() {
    $this->middleware( CRP::role( CRP::P_Cadastro_read ) )->only( [ 'index', 'calc' ] );
    $this->middleware( CRP::role( CRP::P_Cadastro_edit ) )->only( [ 'store', 'edit', 'update', 'destroy' ] );
  }

  /**
   * Retorno o JSON dos eventos da faixa de datas.
   *
   * @param int $id
   * @return string
   */
  public function index( $id ) {
    // verifica se ha os parametros
    if ( !\Request::has( 'start' ) || !\Request::has( 'end' ) ) {
      // ?start=2016-08-28&end=2016-10-09
      return abort( 400, 'Falta Parametros start e end' );
    }

    // procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // os eventos deste Funcionario
    /** @var ControleDePontos $events */
    $events = $func->ControleDePontos();

    // faixa de datas
    $start = \Request::get( 'start' );
    $end   = \Request::get( 'end' );

    // WHERE entra as datas
    $events = $events->whereBetween( 'data', [ $start, $end ] );

    // ordena pela data
    $events = $events->orderBy( 'data' )->get();

    // constroi os array com os dados do evento para o fullCalendar
    $resp = [];
    foreach ( $events as $ev ) {
      $r = [
        'id'              => $ev->id,
        'title'           => ControleDePontos::enum_tipos()[ $ev->tipo ],
        'allDay'          => true,
        'start'           => $ev->data,
        'backgroundColor' => ControleDePontos::enum_tipos_backgroundColor()[ $ev->tipo ],
        'textColor'       => '#000000',
        'url'             => '', //route( 'controledeponto.show', [ $id, $ev->id ] ),
      ];
      array_push( $resp, $r );
    }

    // retorna o array em JSON
    return response()->json( $resp );
  }

  /**
   * Retorna o JSON com o calculo do saldo de hora do mes
   *
   * @param int $id
   * @return \Illuminate\Http\JsonResponse
   */
  public function calc( $id ) {
    // verifica se ha os argumentos
    if ( !\Request::has( 'ano' ) || !\Request::has( 'mes' ) ) { // ?ano=2016&mes=10
      return abort( 400, 'Falta Parametros ano e mes' );
    }

    // procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );

    // formata a margem de data (do dia 01 a 31 do mes)
    $ano = intval( \Request::get( 'ano' ) );
    $mes = intval( \Request::get( 'mes' ) );

    $anoMes = Carbon::create( $ano, $mes, 2, 0, 0, 0 );

    $resp = $func->_CalcControleDePontos( $anoMes );

    // retorna o array em JSON
    return Response::json( $resp );
  }

  /**
   * Insere o evento no BD.
   *
   * @param Request $request
   * @param  int $id
   * @return \Illuminate\Http\JsonResponse
   */
  public function store( Request $request, $id ) {
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );

    if ( $request->get( 'add_lote', false ) ) {
      $data_inicio     = Carbon::createFromFormat( 'Y-m-d', $request->get( 'data_inicio' ) );
      $data_fim        = Carbon::createFromFormat( 'Y-m-d', $request->get( 'data_fim' ) );
      $tipo            = $request->get( 'tipo' );
      $horas           = '08:00';
      $aprovador_id    = $request->get( 'aprovador_id' );
      $obs             = $request->get( 'obs' );

      for ( ; $data_inicio <= $data_fim; $data_inicio->addDay() ) {
        if ( in_array( $data_inicio->dayOfWeek, [ 0, 6 ] ) )
          continue;
        $func->ControleDePontos()->create( [
          'tipo'         => $tipo,
          'data'         => $data_inicio->format( 'Y-m-d' ),
          'horas'        => $horas,
          'aprovador_id' => $aprovador_id,
          'obs'          => $obs,
        ] );
      }
    } else {
      $func->ControleDePontos()->create( $request->all() );
    }

    return Response::json( null, 201 );
  }

  /**
   * Exibe o evento.
   *
   * @param  int $id
   * @param  int $id2
   * @return \Illuminate\Http\JsonResponse
   */
  public function edit( $id, $id2 ) {
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    /** @var ControleDePontos $event */
    $event = $func->ControleDePontos()->findOrFail( $id2 );

    return Response::json( $event );
  }

  /**
   * Atualiza o Evento no BD.
   *
   * @param  Request $request
   * @param  int $id
   * @param  int $id2
   * @return \Illuminate\Http\JsonResponse
   */
  public function update( Request $request, $id, $id2 ) {
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    /** @var ControleDePontos $event */
    $event = $func->ControleDePontos()->findOrFail( $id2 );
    $event->update( $request->all() );

    return Response::json( $event );
  }

  /**
   * Remove o evento do BD.
   *
   * @param  int $id
   * @param  int $id2
   * @return \Illuminate\Http\Response
   */
  public function destroy( $id, $id2 ) {
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    /** @var ControleDePontos $event */
    $event = $func->ControleDePontos()->findOrFail( $id2 );
    $event->delete();

    return response( '', 200 );
  }
}
