<?php

namespace App\Http\Controllers\Cadastros;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Http\Requests\DepartamentoRequest;
use App\Models\Cadastros\Departamentos;
use Illuminate\Database\Eloquent\Builder;
use Request;


class DepartamentosController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Cadastro_read ) )->only( [ 'index', 'show' ] );
    $this->middleware( CRP::role( CRP::P_Cadastro_edit ) )->only( [ 'create', 'store', 'edit', 'update', 'destroy' ] );
  }

  /**
   * Lista os Departamentos.
   *
   * @return \Illuminate\Http\Response
   */
  public function index() {
    // numero de cadastros no BD
    $numTotalDepartCad = Departamentos::count();

    // booleano para marcar se houve alguma pesquisa (utilizado nas msg de total encontrado)
    $pesquisas = false;
    // busca pelo itens pesquisados
    $depart = Departamentos::where( function ( Builder $q ) use ( &$pesquisas ) {
      // campos possiveis de pesquisar
      $campos = [ 'nome' ];
      // para cada campo
      foreach ( $campos as $c ) {
        // se foi atribuido algum valor no $_GET
        if ( Request::has( $c ) ) {
          // adiciona uma clausula WHERE (campo LIKE %valor%)
          $q->where( $c, 'like', '%' . Request::get( $c, '' ) . '%' );
          // houve alguma pesquisa
          $pesquisas = true;
        }
      }
    } )->sortable( [ 'nome' ] )->paginate( 10 );// ordena pela coluna (default eh o nome) e faz paginacao

    return view( 'Cadastros.Departamentos.index', compact( 'depart', 'numTotalDepartCad', 'pesquisas' ) );

  }

  /**
   * Exibe o formulario para inserir novo Departamento.
   *
   * @return \Illuminate\Http\Response
   */
  public function create() {
    return view( 'Cadastros.Departamentos.create' );
  }

  /**
   * Armazena um novo Departamento no BD.
   *
   * @param  DepartamentoRequest $request
   * @return \Illuminate\Http\Response
   */
  public function store( DepartamentoRequest $request ) {
    // Cria o Departamento no BD
    Departamentos::create( $request->all() );

    flash( 'Departamento criado com sucesso!', 'success' );
    return redirect()->route( 'departamentos.index' );
  }

  /**
   * Exibe os atributos do Departamento.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function show( $id ) {
    // procura o Departamentos
    /** @var Departamentos $depart */
    $depart = Departamentos::findOrFail( $id );
    // obtem seus Funcionarios
    $func = $depart->Funcionarios()->orderBy( 'nome' )->get();

    return view( 'Cadastros.Departamentos.show', compact( 'depart', 'func' ) );

  }

  /**
   * Exibe o formulario para editar um Departamento.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function edit( $id ) {
    // procura o Departamentos
    $depart = Departamentos::findOrFail( $id );

    return view( 'Cadastros.Departamentos.edit', compact( 'depart' ) );
  }

  /**
   * Atualiza os dados de um Departamento.
   *
   * @param  DepartamentoRequest $request
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function update( DepartamentoRequest $request, $id ) {
    // procura o Departamentos
    $depart = Departamentos::findOrFail( $id );
    // atualiza seus dados no BD
    $depart->update( $request->all() );

    flash( 'Departamento editado com sucesso!', 'success' );
    return redirect()->route( 'departamentos.index' );
  }

  /**
   * Remove um Departamento do BD.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function destroy( $id ) {
    // procura o Departamentos
    $depart = Departamentos::findOrFail( $id );

    // deleta do banco
    $depart->delete();

    flash( 'Departamento deletado com sucesso!', 'success' );
    return redirect()->route( 'departamentos.index' );
  }
}
