<?php

namespace App\Http\Controllers\Cadastros;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Http\Requests\FuncionarioRequest;
use App\Models\Cadastros\FuncDependentes;
use App\Models\Cadastros\Funcionarios;
use App\Models\Cadastros\FuncTelefones;
use App\Models\Cadastros\Usuarios;
use App\Models\Config\FaixaSalarial;
use App\Notifications\BemVindoNotification;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Image;
use Maatwebsite\Excel\Facades\Excel;
use PHPExcel_Style_Border;
use Request;
use Storage;

class FuncionariosController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Cadastro_read ) )->only( [ 'index', 'show' ] );
    $this->middleware( CRP::role( CRP::P_Cadastro_edit ) )->only( [ 'create', 'store', 'edit', 'update', 'destroy', 'export_contatos_funcionarios_XLSX' ] );
  }

  /**
   * Lista os Funcionarios.
   *
   * @return \Illuminate\Http\Response
   */
  public function index() {
    // numero de cadastros no BD
    $numTotalFuncCad = Funcionarios::count();

    // booleano para marcar se houve alguma pesquisa (utilizado nas msg de total encontrado)
    $pesquisas = false;
    // busca pelo itens pesquisados
    $func = Funcionarios::where( function ( Builder $q ) use ( &$pesquisas ) {
      // campos possiveis de pesquisar
      $campos = [ 'nome', 'depart', 'teste_mbti', 'regime', 'departamentos_id' ];
      // para cada campo
      foreach ( $campos as $c ) {
        // se foi atribuido algum valor no $_GET
        if ( Request::has( $c ) ) {
          // adiciona uma clausula WHERE (campo LIKE %valor%)
          $q->where( $c, 'like', '%' . Request::get( $c, '' ) . '%' );
          // houve alguma pesquisa
          $pesquisas = true;
        }
      }
      // Caso especial Inativo
      if ( !Request::has( 'inativo' ) ) {
        // sempre mostrar os nao inativos por default
        $q->where( 'inativo', '=', '0' );
      } elseif ( Request::get( 'inativo', '' ) != -1 ) {
        // inativo == '', 0 ou 1
        $q->where( 'inativo', 'like', '%' . Request::get( 'inativo', '' ) . '%' );
        $pesquisas = true;
      } else {
        // inativo= -1
        $pesquisas = true;
      }
    } )->sortable( [ 'nome' ] )->paginate( 10 );// ordena pela coluna (default eh o nome) e faz paginacao

    return view( 'Cadastros.Funcionarios.index', compact( 'func', 'numTotalFuncCad', 'pesquisas' ) );
  }

  /**
   * Exibe o formulario para inserir novo Funcionario.
   *
   * @return \Illuminate\Http\Response
   */
  public function create() {
    $telefones   = [];
    $dependentes = [];
    $ehCreate    = true;

    return view( 'Cadastros.Funcionarios.create', compact( 'telefones', 'dependentes', 'ehCreate' ) );
  }

  /**
   * Armazena um novo Funcionario no BD.
   *
   * @param  FuncionarioRequest $request
   * @return \Illuminate\Http\Response
   */
  public function store( FuncionarioRequest $request ) {

    // ============ Dados Basicos do Funcionario ============
    // Cria o Funcionario no BD
    /** @var Funcionarios $func */
    $func = Funcionarios::create( $request->all() );

    // ============ Foto do Funcionario ============
    if ( $request->hasFile( 'profile_image' ) ) { // ha foto?
      // get a foto
      $file = $request->file( 'profile_image' );
      // o nome sera o ID do Funcionario e a extensao original. Ex: {id}.jpg
      $filename = $func->getAttribute( 'id' ) . '.' . $file->getClientOriginalExtension();
      // sua localizacao sera /public/images/profiles/
      $location = public_path( 'images/profiles/' . $filename );
      // salva a imagem
      Image::make( $file )->resize( 230, 230 )->save( $location );

      // Salva o valor do caminho da foto no DB
      $func->foto = $filename;
      $func->save();
    }


    // ============ Insere os Telefones ============
    // get do POST
    $tel_tipo     = array_key_exists( 'tel_tipo', $request->all() ) ? $request->all()[ 'tel_tipo' ] : [];
    $tel_telefone = array_key_exists( 'tel_telefone', $request->all() ) ? $request->all()[ 'tel_telefone' ] : [];
    $tel_obs      = array_key_exists( 'tel_obs', $request->all() ) ? $request->all()[ 'tel_obs' ] : [];
    $tel_recado   = array_key_exists( 'tel_recado', $request->all() ) ? $request->all()[ 'tel_recado' ] : [];

    // insere os telefones dele
    for ( $i = 0; $i < count( $tel_tipo ); $i++ ) {
      // cria novo telefone
      $func->Telefones()->create(
        [ 'tipo'     => $tel_tipo[ $i ],
          'telefone' => $tel_telefone[ $i ],
          'recado'   => $tel_recado[ $i ],
          'obs'      => $tel_obs[ $i ] == '' ? null : $tel_obs[ $i ]
        ]
      );
    }

    // ============ Insere os Dependentes ============
    // get do POST
    $dep_nome       = array_key_exists( 'dep_nome', $request->all() ) ? $request->all()[ 'dep_nome' ] : [];
    $dep_parentesco = array_key_exists( 'dep_parentesco', $request->all() ) ? $request->all()[ 'dep_parentesco' ] : [];
    $dep_nascimento = array_key_exists( 'dep_nascimento', $request->all() ) ? $request->all()[ 'dep_nascimento' ] : [];
    $dep_falecido   = array_key_exists( 'dep_falecido', $request->all() ) ? $request->all()[ 'dep_falecido' ] : [];

    // insere os dependentes dele
    for ( $i = 0; $i < count( $dep_nome ); $i++ ) {
      // cria novo dependent
      $func->Dependentes()->create( [
          'nome'            => $dep_nome[ $i ],
          'parentesco'      => $dep_parentesco[ $i ],
          'data_nascimento' => $dep_nascimento[ $i ],
          'falecido'        => $dep_falecido[ $i ],
        ]
      );
    }

    // deseja tbm add ele como usuario?
    if ( $request->get( 'usar_usuario', 0 ) == 1 ) {
      // Criar Usuario no BD
      $senha = str_random( 10 );
      /** @var Usuarios $user */
      $user = Usuarios::create( [
        'name'            => $func->nome,
        'email'           => $func->email,
        'password'        => bcrypt( $senha ),
        'crp_role'        => $request->get( 'crp_role', 0 ) == '' ? 0 : $request->get( 'crp_role', 0 ),
        'funcionarios_id' => $func->id
      ] );
      $user->ConfigRole();

      // envia a senha por email
      $user->notify( new BemVindoNotification( $user->email, $senha ) );
    }

    // deseja tbm add historico de contratação
    if ( $request->get( 'usar_contratacao', 0 ) == 1 ) {
      // Criar Historico de contratacao
      $hist = [
        'data'        => $request->get( 'hist_data' ),
        'tipo'        => 0,
        'cargo'       => $request->get( 'hist_cargo', null ),
        'nivel'       => $request->get( 'hist_nivel', null ),
        'salario'     => $request->get( 'hist_salario', null ),
        'vr'          => $request->get( 'hist_vr', null ),
        'vt'          => $request->get( 'hist_vt', null ),
        'seguro_vida' => $request->get( 'hist_seguro_vida', null ),
      ];
      $func->Historicos()->create( $hist );
    }

    flash( 'Funcionário criado com sucesso!', 'success' );
    return redirect()->route( 'funcionarios.index' );

  }

  /**
   * Exibe os atributos do Funcionario.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function show( $id ) {
    // procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // obtem seus Telefones
    $telefones = $func->Telefones()->orderBy( 'recado' )->get();
    // obtem seus Dependentes
    $dependentes = $func->Dependentes()->get();
    // obter sua situacao atual
    $hist = $func->pilhaHistorico();

    return view( 'Cadastros.Funcionarios.show', compact( 'func', 'telefones', 'dependentes', 'hist' ) );
  }

  /**
   * Exibe o formulario para editar um Funcionario.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function edit( $id ) {
    // procura o funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // obtem seus Telefones
    $telefones = $func->Telefones()->get();
    // obtem seus Dependentes
    $dependentes = $func->Dependentes()->get();
    // nao eh o create (usado qdo cria o Funcionario pela primeira vez, e ale atrelar User e Contratacao
    $ehCreate = false;

    return view( 'Cadastros.Funcionarios.edit', compact( 'func', 'telefones', 'dependentes', 'ehCreate' ) );
  }

  /**
   * Atualiza os dados de um Funcionario.
   *
   * @param  FuncionarioRequest $request
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function update( FuncionarioRequest $request, $id ) {
    // encontra o Funcionario Original
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );


    // ============ Dados Basicos do Funcionario ============
    // atualiza seus dados no BD
    $func->update( $request->all() );


    // ============ Foto do Funcionario ============
    if ( $request->hasFile( 'profile_image' ) ) { // ha foto?
      if ( $func->foto != null ) {
        // deleta a antiga se tiver
        Storage::delete( 'images/profiles/' . $func->foto );
      }

      // get a foto
      $file = $request->file( 'profile_image' );
      // o nome sera o ID do Funcionario e a extensao original. Ex: {id}.jpg
      $filename = $func->getAttribute( 'id' ) . '.' . $file->getClientOriginalExtension();
      // sua localizacao sera /public/images/profiles/
      $location = public_path( 'images/profiles/' . $filename );
      // salva a imagem
      Image::make( $file )->resize( 230, 230 )->save( $location );

      // atualiza o valor do caminho da foto no DB
      $func->foto = $filename;
      $func->save();
    }


    // ============ Atualiza os Telefones ============
    // get do POST
    $tel_id       = array_key_exists( 'tel_id', $request->all() ) ? $request->all()[ 'tel_id' ] : [];
    $tel_tipo     = array_key_exists( 'tel_tipo', $request->all() ) ? $request->all()[ 'tel_tipo' ] : [];
    $tel_telefone = array_key_exists( 'tel_telefone', $request->all() ) ? $request->all()[ 'tel_telefone' ] : [];
    $tel_obs      = array_key_exists( 'tel_obs', $request->all() ) ? $request->all()[ 'tel_obs' ] : [];
    $tel_recado   = array_key_exists( 'tel_recado', $request->all() ) ? $request->all()[ 'tel_recado' ] : [];

    // funcao usada no array_map para selecionar o 'id', e tornar o array associativo em somente array
    $soID = function ( $value ) { return $value[ 'id' ]; };

    // analisa as diferencas
    // flat a lista de ids, ex: [1, 2, 3, 4]
    $tel_antigo = array_map( $soID, $func->Telefones()->get( [ 'id' ] )->toArray() );
    // calcula a diferença ( encontra os valores nao presente no $tel_id, mas que tinha no antigo) e reinicia o array associativo
    $tel_deletados = array_values( array_diff( $tel_antigo, $tel_id ) );

    // deleta os anteriores do BD
    FuncTelefones::destroy( $tel_deletados );

    // insere os novos ou atualiza os editados
    for ( $i = 0; $i < count( $tel_tipo ); $i++ ) {
      // array com os dados
      $tel = [
        'tipo'     => $tel_tipo[ $i ],
        'telefone' => $tel_telefone[ $i ],
        'recado'   => $tel_recado[ $i ],
        'obs'      => $tel_obs[ $i ] == '' ? null : $tel_obs[ $i ] ];
      if ( $tel_id[ $i ] == '-1' ) { // cria novo telefone
        $func->Telefones()->create( $tel );
      } else { // atualiza telefone com aquele id
        $func->Telefones()->find( $tel_id[ $i ] )->update( $tel );
      }
    }


    // ============ Atualiza os Dependentes ============
    // get do POST
    $dep_id         = array_key_exists( 'dep_id', $request->all() ) ? $request->all()[ 'dep_id' ] : [];
    $dep_nome       = array_key_exists( 'dep_nome', $request->all() ) ? $request->all()[ 'dep_nome' ] : [];
    $dep_parentesco = array_key_exists( 'dep_parentesco', $request->all() ) ? $request->all()[ 'dep_parentesco' ] : [];
    $dep_nascimento = array_key_exists( 'dep_nascimento', $request->all() ) ? $request->all()[ 'dep_nascimento' ] : [];
    $dep_falecido   = array_key_exists( 'dep_falecido', $request->all() ) ? $request->all()[ 'dep_falecido' ] : [];

    // Analisa as diferencas
    // flat a lista de ids, ex: [1, 2, 3, 4]
    $dep_antigos = array_map( $soID, $func->Dependentes()->get( [ 'id' ] )->toArray() );
    // calcula a diferença ( encontra os valores nao presente no $dep_id, mas que tinha no antigo) e reinicia o array associativo
    $dep_deletados = array_values( array_diff( $dep_antigos, $dep_id ) );

    // deleta os anteriores do BD
    FuncDependentes::destroy( $dep_deletados );

    // insere os novos ou atualiza os editados
    for ( $i = 0; $i < count( $dep_nome ); $i++ ) {
      // array com os dados
      $dep = [
        'nome'            => $dep_nome[ $i ],
        'parentesco'      => $dep_parentesco[ $i ],
        'data_nascimento' => $dep_nascimento[ $i ],
        'falecido'        => $dep_falecido[ $i ],
      ];
      if ( $dep_id[ $i ] == '-1' ) { // cria novo dependente
        $func->Dependentes()->create( $dep );
      } else { // atualiza dependente com aquele id
        $func->Dependentes()->find( $dep_id[ $i ] )->update( $dep );
      }
    }

    flash( 'Funcionário editado com sucesso!', 'success' );
    return redirect()->route( 'funcionarios.index' );
  }

  /**
   * Remove um Funcionario do BD.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function destroy( $id ) {
    // procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );

    // apaga a foto do perfil do public
    if ( $func->foto != null ) {
      Storage::delete( 'images/profiles/' . $func->foto );
    }

    // deleta do banco
    $func->delete();

    flash( 'Funcionário deletado com sucesso!', 'success' );
    return redirect()->route( 'funcionarios.index' );
  }

  /**
   * Exibe o Relatorio de Salarios dos Funcionarios
   *
   * @return \Illuminate\Http\Response
   */
  public function relSalarial() {
    $fs = Funcionarios::where( 'inativo', '0' )->orderby( 'nome' )->get();

    $faixas   = FaixaSalarial::getfaixas();
    $calcPorc = function ( $sal, $min, $max ) {
      if ( $sal == null || $max == null || $min == null ) {
        return 0;
      }
      $x = intval( ( ( $sal - $min ) * 100 ) / ( $max - $min ) );
      if ( $x < 0 ) {
        return 0;
      } elseif ( $x > 100 ) {
        return 100;
      }
      return $x;
    };

    $funcs = [];
    foreach ( $fs as $f ) {
      $h = $f->pilhaHistorico();
      if ( $h[ 'nivel' ] == null ) {
        $funcs[] = [
          'id'       => $f->id,
          'nome'     => $f->nome,
          'nivel'    => 0,
          'salario'  => 0,
          'porc'     => 0,
          'dissidio' => false
        ];
      } else {
        $faixa   = $faixas[ $h[ 'nivel' ] ];
        $funcs[] = [
          'id'       => $f->id,
          'nome'     => $f->nome,
          'nivel'    => $h[ 'nivel' ],
          'salario'  => $h[ 'salario' ],
          'porc'     => $calcPorc( $h[ 'salario' ], $faixa[ 'inferior' ], $faixa[ 'superior' ] ),
          'dissidio' => $f->jaTeveDissidioEsteAno()
        ];
      }
    }

    return view( 'Cadastros.Funcionarios.relSalarial', compact( 'funcs' ) );
  }

  /**
   * Exibe o Relatorio de Ferias dos Funcionarios
   *
   * @return \Illuminate\Http\Response
   */
  public function relFerias() {
    $fs = Funcionarios::where( 'inativo', '0' )->orderby( 'nome' )->get();

    $today    = new Carbon();
    $calcPorc = function ( Carbon $inicio, Carbon $fim ) use ( $today ) {
      if ( $inicio == null || $fim == null ) {
        return 0;
      }
      $diff_IF = $inicio->diff( $fim, true );
      $diff_IF = ( $diff_IF->m + ( $diff_IF->d > 29 ? 1 : 0 ) ) + $diff_IF->y * 12;
      $diff_IT = $inicio->diff( $today, true );
      $diff_IT = ( $diff_IT->m + ( $diff_IT->d > 29 ? 1 : 0 ) ) + $diff_IT->y * 12;

      $x = intval( ( ( $diff_IT ) * 100 ) / ( $diff_IF ) );
      if ( $x < 0 ) {
        return 0;
      } elseif ( $x > 100 ) {
        return 100;
      }
      return $x;
    };

    $funcs = [];
    foreach ( $fs as $f ) {
      $h = $f->pilhaHistorico();
      if ( $h[ 'aquisitivo_inicio' ] == null || $h[ 'aquisitivo_termino' ] == null ) {
        $funcs[] = [
          'id'         => $f->id,
          'nome'       => $f->nome,
          'aquisitivo' => ' - ',
          'meses'      => '-',
          'dias'       => '-',
          'porc'       => 0
        ];
      } else {
        $inicio  = Carbon::createFromFormat( 'Y-m-d', $h[ 'aquisitivo_inicio' ] );
        $fim     = Carbon::createFromFormat( 'Y-m-d', $h[ 'aquisitivo_termino' ] );
        $meses   = $inicio->diff( $today, true );
        $meses   = ( $meses->m + ( $meses->d > 29 ? 1 : 0 ) ) + $meses->y * 12;
        $funcs[] = [
          'id'         => $f->id,
          'nome'       => $f->nome,
          'aquisitivo' => $inicio->format( 'd/m/Y' ) . ' à ' . $fim->format( 'd/m/Y' ),
          'meses'      => $meses,
          'dias'       => $meses * 2.5,
          'porc'       => $calcPorc( $inicio, $fim )
        ];
      }
    }

    return view( 'Cadastros.Funcionarios.relFerias', compact( 'funcs' ) );
  }

  /**
   * Gera o XLSX dos Contatos dos Funcionarios.
   *
   * @return void
   */
  public function export_contatos_funcionarios_XLSX() {
    /** @var Funcionarios[] $funcionarios */
    $funcionarios = Funcionarios::orderBy( 'nome' )->get();

    $XLSX   = [];
    $XLSX[] = [
      'NOME',
      'ENDEREÇO',
      'EMAIL',
      'TELEFONE',
      'REGIME',
      'SALÁRIO',
      'MBTI',
    ];
    foreach ( $funcionarios as $funcionario ) {
      $salario = $funcionario->_SalarioAtual();
      $XLSX[]  = [
        $funcionario->nome,
        $funcionario->_EnderecoCompleto(),
        $funcionario->email,
        $funcionario->getUmTelefone(),
        $funcionario->getRegimeString(),
        // number_format( intval( $salario ) / 100, 2, ',', '.' ),
        intval( $salario ) / 100,
        $funcionario->_MBTI(),
      ];
    }

    Excel::create( 'Sistema Cadaris - Contatos dos Funcionários', function ( $excel ) use ( $XLSX ) {
      /** @var \Maatwebsite\Excel\Writers\LaravelExcelWriter $excel */
      $excel->setTitle( 'Sistema Cadaris - Contatos dos Funcionários' );
      $excel->setDescription( 'Sistema Cadaris - Contatos dos Funcionários' );
      $excel->sheet( 'Contatos dos Funcionários', function ( $sheet ) use ( $XLSX ) {
        /** @var \Maatwebsite\Excel\Classes\LaravelExcelWorksheet $sheet */
        // Dados
        $sheet->fromArray( $XLSX, null, 'A1', false, false );

        // Style geral
        $sheet->setStyle( [
          'font' => [
            'name' => 'Arial',
            'size' => 10
          ]
        ] );
        $sheet->setAutoSize( true );
        $sheet->setBorder( 'A1:G' . count( $XLSX ), PHPExcel_Style_Border::BORDER_THIN );

        // Style do Header
        $sheet->setHeight( 1, 25 );
        $sheet->cells( 'A1:G1', function ( $cells ) {
          /** @var \Maatwebsite\Excel\Writers\CellWriter $cells */
          $cells->setFontColor( '#ffffff' );
          $cells->setBackground( '#404040' );
          $cells->setAlignment( 'center' );
          $cells->setValignment( 'center' );
          $cells->setFontWeight( 'bold' );
        } );

        // Auto Filter
        $sheet->setAutoFilter();
      } );
    } )->download( 'xlsx' );
  }
}
