<?php

namespace App\Http\Controllers\Cadastros;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Http\Requests;
use App\Http\Requests\FuncionarioHistoricoLoteRequest;
use App\Http\Requests\FuncionarioHistoricoRequest;
use App\Models\Cadastros\FuncHistoricos;
use App\Models\Cadastros\Funcionarios;

class FuncionariosHistoricosController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Cadastro_read ) )->only( [ 'index' ] );
    $this->middleware( CRP::role( CRP::P_Cadastro_edit ) )->only( [ 'create', 'store', 'edit', 'update', 'destroy', 'getHistoricoLote', 'postHistoricoLote' ] );
  }

  /**
   * Lista o Historico do Funcionario.
   *
   * @param  integer $id
   * @return \Illuminate\Http\Response
   */
  public function index( $id ) {
    // procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // obtem seu Historico
    $funcHist = $func->Historicos()->orderBy( 'data', 'DESC' )->get();
    return view( 'Cadastros.FuncHistorico.index', compact( 'func', 'funcHist' ) );
  }

  /**
   * Exibe o formulario para criar um novo Historico do Funcionario.
   *
   * @param  integer $id
   * @return \Illuminate\Http\Response
   */
  public function create( $id ) {
    // Procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );

    return view( 'Cadastros.FuncHistorico.create', compact( 'func' ) );
  }

  /**
   * Armazena um novo Historico do Funcionario no BD.
   *
   * @param  integer $id
   * @param  Requests\FuncionarioHistoricoRequest $request
   * @return \Illuminate\Http\Response
   */
  public function store( FuncionarioHistoricoRequest $request, $id ) {
    // get do POST, set null nos vazios ('') e retorna um array
    $hist = FuncHistoricos::setNulls( $request->all() );
    // encontra o Funcionario destes Historicos
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // Cria o Historico para este Funcionario no BD
    $func->Historicos()->create( $hist );

    flash( 'Histórico criado com sucesso!', 'success' );
    return redirect()->route( 'historicos.index', [ $id ] );

  }

  /**
   * Exibe o formulario para editar um Historico do Funcionario.
   *
   * @param  int $id
   * @param  int $historicos
   * @return \Illuminate\Http\Response
   */
  public function edit( $id, $historicos ) {
    // procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // procura um Historico deste Funcionario
    $funcHist = $func->Historicos()->find( $historicos );

    return view( 'Cadastros.FuncHistorico.edit', compact( 'func', 'funcHist' ) );
  }

  /**
   * Atualiza os dados de um Historico do Funcionario no BD.
   *
   * @param  FuncionarioHistoricoRequest $request
   * @param  int $id
   * @param int $historicos
   * @return \Illuminate\Http\Response
   */
  public function update( FuncionarioHistoricoRequest $request, $id, $historicos ) {
    // get do POST, set null nos vazios ('') e retorna um array
    $hist = FuncHistoricos::setNulls( $request->all() );
    // encontra o Funcionario destes Historicos
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // encontra o Historico original
    $funcHist = $func->Historicos()->find( $historicos );
    // atualiza seus dados no BD
    $funcHist->update( $hist );

    flash( 'Histórico editado com sucesso!', 'success' );
    return redirect()->route( 'historicos.index', [ $id ] );
  }

  /**
   * Deleta um Historico do Funcionario do BD.
   *
   * @param  $id
   * @param int $historicos
   * @return \Illuminate\Http\Response
   */
  public function destroy( $id, $historicos ) {
    // procura o Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::findOrFail( $id );
    // encontra este historico dele
    $hist = $func->Historicos()->find( $historicos );
    // deleta do banco
    $hist->delete();

    flash( 'Histórico deletado com sucesso!', 'success' );
    return redirect()->route( 'historicos.index', [ $id ] );
  }

  /**
   * Lista os Funcionarios e o formulario para add um Historico em lote.
   *
   * @return \Illuminate\Http\Response
   */
  public function getHistoricoLote() {
    // Lista os Funcionario
    /** @var Funcionarios $func */
    $func = Funcionarios::orderBy( 'nome' )->get( [ 'id', 'nome' ] );

    return view( 'Cadastros.FuncHistorico.lote', compact( 'func' ) );
  }

  /**
   * Atribui um Historico aos Funcionarios.
   *
   * @param FuncionarioHistoricoLoteRequest $request
   * @return \Illuminate\Http\Response
   */
  public function postHistoricoLote( FuncionarioHistoricoLoteRequest $request ) {
    // get os ids dos funcionarios escolhidos
    $ids = $request->all()[ 'idFunc' ];
    // procura os Funcionarios
    /** @var Funcionarios[] $funcs */
    $funcs = Funcionarios::findMany( $ids );
    // limpar os campos para ser utilizado no BD
    $hist = FuncHistoricos::setNulls( $request->all() );
    // para cada funcionario
    foreach ( $funcs as $f ) {
      // se for dissidio
      if ( $hist[ 'tipo' ] == 3 ) {
        // calcula para ele, e se é possivel add dissidio
        $diss = $f->calcDissidio();
        // se deu OK, entao aplica o salario, senao nem criara o historico
        if ( $diss[ 'codigo' ] == 1 ) {
          // cria o historico
          /** @var FuncHistoricos $h */
          $h = $f->Historicos()->create( $hist );
          // edita o salario
          $h->salario = $diss[ 'salAjust' ];
          // salva
          $h->save();
        }
      } else { // sao outros tipos
        // add o historico a ele
        $f->Historicos()->create( $hist );
      }
    }

    flash( 'Histórico adicionado em lote com sucesso aos Funcionários selecionados!', 'success' );
    return redirect()->route( 'funcionarios.index' );
  }
}
