<?php

namespace App\Http\Controllers\Cadastros;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Http\Requests\UsuarioRequest;
use App\Models\Cadastros\Usuarios;
use App\Notifications\BemVindoNotification;
use Illuminate\Database\Eloquent\Builder;
use Request;

class UsuariosController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Cadastro_read ) )->only( [ 'index', 'show' ] );
    $this->middleware( CRP::role( CRP::P_Cadastro_edit ) )->only( [ 'create', 'store', 'edit', 'update', 'destroy' ] );
  }

  /**
   * Lista os Usuarios.
   *
   * @return \Illuminate\Http\Response
   */
  public function index() {
    // numero de cadastros no BD
    $numTotalUsuariosCad = Usuarios::count();

    // booleano para marcar se houve alguma pesquisa (utilizado nas msg de total encontrado)
    $pesquisas = false;
    // busca pelo itens pesquisados
    $users = Usuarios::where( function ( Builder $q ) use ( &$pesquisas ) {
      // campos possiveis de pesquisar
      $campos = [ 'name' ];
      // para cada campo
      foreach ( $campos as $c ) {
        // se foi atribuido algum valor no $_GET
        if ( Request::has( $c ) ) {
          // adiciona uma clausula WHERE (campo LIKE %valor%)
          $q->where( $c, 'like', '%' . Request::get( $c, '' ) . '%' );
          // houve alguma pesquisa
          $pesquisas = true;
        }
      }
    } )->sortable( [ 'name' ] )->paginate( 10 );// ordena pela coluna (default eh o nome) e faz paginacao

    return view( 'Cadastros.Usuarios.index', compact( 'users', 'numTotalUsuariosCad', 'pesquisas' ) );

  }

  /**
   * Exibe o formulario para inserir novo Departamento.
   *
   * @return \Illuminate\Http\Response
   */
  public function create() {
    return view( 'Cadastros.Usuarios.create' );
  }

  /**
   * Armazena um novo Usuario no BD.
   *
   * @param  UsuarioRequest $request
   * @return \Illuminate\Http\Response
   */
  public function store( UsuarioRequest $request ) {
    // Get os dados do POST
    $dados = $request->all();
    // crypt a senha
    $senha               = str_random( 10 );
    $dados[ 'password' ] = bcrypt( $senha );
    // Cria o Usuario no BD
    /** @var Usuarios $user */
    $user = Usuarios::create( $dados );
    $user->ConfigRole();

    // envia a senha por email
    $user->notify( new BemVindoNotification( $user->email, $senha ) );

    flash( 'Usuário criado com sucesso!', 'success' );
    return redirect()->route( 'usuarios.index' );
  }

  /**
   * Exibe os atributos do Usuario.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function show( $id ) {
    // procura o Usuario
    /** @var Usuarios $user */
    $user = Usuarios::findOrFail( $id );
    // obtem seu Funcionario associado
    $func = $user->Funcionario;

    return view( 'Cadastros.Usuarios.show', compact( 'user', 'func' ) );

  }

  /**
   * Exibe o formulario para editar um Usuario.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function edit( $id ) {
    // procura o Usuario
    /** @var Usuarios $user */
    $user     = Usuarios::findOrFail( $id );
    $nomeFunc = $user->Funcionario->nome;

    return view( 'Cadastros.Usuarios.edit', compact( 'user', 'nomeFunc' ) );
  }

  /**
   * Atualiza os dados de um Usuario.
   *
   * @param  UsuarioRequest $request
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function update( UsuarioRequest $request, $id ) {
    // procura o Usuario
    /** @var Usuarios $user */
    $user = Usuarios::findOrFail( $id );

    // atualiza seus dados no BD
    $user->update( $request->all() );
    $user->ConfigRole();

    flash( 'Usuário editado com sucesso!', 'success' );
    return redirect()->route( 'usuarios.index' );
  }

  /**
   * Remove um Departamento do BD.
   *
   * @param  int $id
   * @return \Illuminate\Http\Response
   */
  public function destroy( $id ) {
    // procura o Usuario
    /** @var Usuarios $user */
    $user = Usuarios::findOrFail( $id );

    // deleta do banco
    $user->delete();

    flash( 'Usuário deletado com sucesso!', 'success' );
    return redirect()->route( 'usuarios.index' );
  }
}
