<?php

namespace App\Http\Controllers\Comercial;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Http\Requests\JobRequest;
use App\Models\Comercial\Campanhas;
use App\Models\Comercial\JobLogs;
use App\Models\Comercial\Jobs;
use App\Models\Config\ServicoCustoInterno;
use Illuminate\Database\Eloquent\Builder;
use Request;

class JobsController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Comercial_index ) )->only( [ 'index' ] );
    $this->middleware( CRP::role( CRP::P_Comercial_0_edit ) )->only( [ 'create', 'store' ] );
    $this->middleware( CRP::role( CRP::P_Comercial_L_edit ) )->only( [ 'destroy' ] );
    // destroy?
    // campanhas?
    // servicos do CI?
  }

  /**
   * Lista todos os Jobs por etapas.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
   */
  public function index() {
    $allJobs = [];
    $allPerm = [];
    foreach ( [ 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'Z', 'L' ] as $stage ) {
      $allPerm[ $stage ] = \Auth::user()->can( CRP::enum_P_comercial_read[ $stage ] );
      $allJobs[ $stage ] =
        Jobs::where( 'etapa_atual', '=', $stage )
            ->where( function ( Builder $q ) {
              $campos = [ 'responsavel_id', 'cliente_cdc_id' ];
              foreach ( $campos as $campo ) {
                if ( Request::has( $campo ) ) {
                  $q->where( $campo, '=', Request::get( $campo, '' ) );
                }
              }
              if ( Request::has( 'nome_job' ) ) {
                $q->where( 'nome_job', 'like', '%' . Request::get( 'nome_job', '' ) . '%' );
                $q->orWhere( 'id', 'like', '%' . Request::get( 'nome_job', '0' ) . '%' );
              }
            } )->sortable( [ 'id' ] )->paginate( 5, [ '*' ], 'page' . $stage );
    }

    return view( 'Comercial.JobIndex', compact( 'allJobs', 'allPerm' ) );
  }

  /**
   * Mostra o formulario para criar um novo Job.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
   */
  public function create() {
    $func        = \Auth::user()->Funcionario;
    $nomeUsuario = 'USUÁRIO SEM FUNCIONÁRIO';
    if ( $func ) {
      $nomeUsuario = $func->nome;
    }

    return view( 'Comercial.create', compact( 'nomeUsuario' ) );
  }

  /**
   * Armazena o novo Job e redireciona para o show.
   *
   * @param JobRequest $request
   * @return \Illuminate\Http\RedirectResponse
   */
  public function store( JobRequest $request ) {
    $job = StageUtil::NOVO( $request );
    StageUtil::LOG( $job, StageUtil::A, JobLogs::LOG_AVANCAR );

    return redirect()->route( 'jobs.show', [ $job->id ] );
  }

  /**
   * Visualiza um Job.
   *
   * @param $id
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
   */
  public function show( $id ) {
    /** @var Jobs $job */
    $job   = Jobs::findOrFail( $id );
    $stage = StageUtil::getSTAGE( $job );
    if ( !Request::user()->can( $stage->PERMISSION_read ) ) {
      return redirect()->route( 'jobs.index' );
    }

    return $stage->showDados();
  }

  /**
   * Atualiza o Job.
   *
   * @param JobRequest $request
   * @param integer $id
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Http\RedirectResponse|\Illuminate\View\View|string
   */
  public function update( JobRequest $request, $id ) {
    /** @var Jobs $job */
    $job   = Jobs::findOrFail( $id );
    $stage = StageUtil::getSTAGE( $job );
    if ( !Request::user()->can( $stage->PERMISSION_edit ) ) {
      return redirect()->route( 'jobs.index' );
    }

    $sub = $request->get( 'tipo_submit', null );
    if ( $sub === 'save' ) {
      flash( 'Proposta Comercial Salva com sucesso!', 'success' );
      return $stage->saveDados( $request );

    } elseif ( $sub === 'send' ) {
      flash( 'Proposta Comercial Avançada com sucesso!', 'success' );
      return $stage->submitDados( $request );

    } elseif ( $sub === 'back' ) {
      flash( 'Proposta Comercial Retornada com sucesso!', 'success' );
      return $stage->back( $request );

    } else {
      return dd( 'jobController->update() erro: BTN_submit' );
    }
  }

  /**
   * Delete um Job.
   *
   * @param integer $id
   * @return \Illuminate\Http\RedirectResponse
   */
  public function destroy( $id ) {
    /** @var Jobs $job */
    $job = Jobs::findOrFail( $id );
    // $job->delete();

    $stages = $job->Stages();
    $stages->delete();
    $job->etapa_atual = 'L';
    $job->save();
    StageUtil::LOG( $job, StageUtil::L, JobLogs::LOG_AVANCAR );

    flash( 'JOB Enviado para Lixeira com sucesso!', 'success' );
    return redirect()->route( 'jobs.index' );
  }

  /**
   * Lista as Campanhas.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
   */
  public function campanhasIndex() {
    $campanhas      = Campanhas::orderBy( 'nome' )->get();
    $campanhasCount = $campanhas->count();

    return view( 'Comercial.campanhas', compact( 'campanhas', 'campanhasCount' ) );
  }

  /**
   * Cria uma nova Campanha.
   *
   * @return \Illuminate\Http\RedirectResponse
   */
  public function campanhasCreate() {
    Campanhas::create( Request::all() );

    flash( 'Campanha criada com sucesso!', 'success' );
    return redirect()->route( 'jobs.campanhasIndex' );
  }

  /**
   * Delete uma Campanha.
   *
   * @param integer $id
   * @return \Illuminate\Http\RedirectResponse
   */
  public function campanhasDestroy( $id ) {
    $campanha = Campanhas::findOrFail( $id );
    $campanha->delete();

    flash( 'Campanha deletada com sucesso!', 'success' );
    return redirect()->route( 'jobs.campanhasIndex' );
  }

  /**
   * Lista os Servicos do Custo Interno.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
   */
  public function ServicoCustoInternoIndex() {
    $servicos      = ServicoCustoInterno::orderBy( 'nome' )->get();
    $servicosCount = $servicos->count();

    return view( 'Comercial.servicoCustoInterno', compact( 'servicos', 'servicosCount' ) );
  }

  /**
   * Cria um Servico do Custo Interno.
   *
   * @return \Illuminate\Http\RedirectResponse
   */
  public function ServicoCustoInternoCreate() {
    ServicoCustoInterno::create( Request::all() );

    flash( 'Servico de Custo Interno criado com sucesso!', 'success' );
    return redirect()->route( 'jobs.servicoCustoInternoIndex' );
  }

  /**
   * Deleta um Servico do Custo Interno.
   *
   * @param $id
   * @return \Illuminate\Http\RedirectResponse
   */
  public function ServicoCustoInternosDestroy( $id ) {
    $servicos = ServicoCustoInterno::findOrFail( $id );
    $servicos->delete();

    flash( 'Servico de Custo Interno deletado com sucesso!', 'success' );
    return redirect()->route( 'jobs.servicoCustoInternoIndex' );
  }

}

/*

0 -(Start do JOB) Preenchimento dos dados

A - Briefing (Form&Anexos)
      livre
      impresso
      digitais
      produção
      anexo
  - Cronograma

B - Custos - Orçamento
      Descricao
      Custos Internos
      Custos de Operacionalização
  - Prazo/Condição de Pagamento

C - Revisa e Gera PDF preliminar dos custos
  - Diretoria Aprova?
  - Gera PDF final

D - Atendente recebe PDF (cc diretoria)
  - Cliente Aprova?

Z - Arquivado














 */
