<?php

namespace App\Http\Controllers\Comercial;

use App\CRP;
use App\Http\Requests\JobRequest;
use App\Models\Cadastros\Funcionarios;
use App\Models\Comercial\Jobs;
use App\Models\Comercial\Stages;
use App\Notifications\JobStageNotification;
use Auth;
use Carbon\Carbon;
use Notification;

class StageUtil {

  const A = 'A';
  const B = 'B';
  const C = 'C';
  const D = 'D';
  const E = 'E';
  const F = 'F';
  const G = 'G';
  const Z = 'Z';
  const L = 'L';

  /**
   * Armazena um novo Job.
   *
   * @param JobRequest $request
   * @return Jobs
   */
  public static function NOVO( JobRequest $request ) {
    $previsao_entrega = sprintf( '%s %s', $request->getOrNull( 'previsao_entrega_date', '' ), $request->getOrNull( 'previsao_entrega_time', '' ) );
    if ( trim( $previsao_entrega ) == '' ) {
      $previsao_entrega = null;
    } else {
      $previsao_entrega .= ':00';
    }

    /** @var Jobs $job */
    $job = Jobs::create( [
      'etapa_atual'       => 'A',
      'nome_job'          => $request->getOrNull( 'nome_job', 'NOME REQUERIDO!!!' ),
      'campanha_id'       => $request->getOrNull( 'campanha_id' ),
      'responsavel_id'    => \Auth::user()->funcionarios_id,
      'cliente_unidd_id'  => $request->getOrNull( 'cliente_unidd_id' ),
      'cliente_cdc_id'    => $request->getOrNull( 'cliente_cdc_id' ),
      'cod_receita'       => $request->getOrNull( 'cod_receita' ),
      'descricao_evento'  => $request->getOrNull( 'descricao_evento' ),
      'previsao_proposta' => $request->getOrNull( 'previsao_proposta' ),
      'previsao_entrega'  => $previsao_entrega
    ] );

    $job->Stages()->create( [
      'etapa'   => 'A',
      'entrada' => Carbon::now()->format( 'Y-m-d H:i:s' ),
      'salvo'   => null,
      'saida'   => null
    ] );

    StageUtil::sendEmail( $job, CRP::P_Comercial_email_A_entrou );
    return $job;
  }

  /**
   * Envia uma notificacao por email para os responsaveis pelo etapa do job.
   *
   * @param Jobs $job
   * @param string $tipo
   */
  public static function sendEmail( Jobs $job, $tipo ) {
    $users = CRP::getUsersFromPermission( $tipo );
    Notification::send( $users, new JobStageNotification( $job, $tipo ) );
  }

  /**
   * Retorna o etapa Controller atual do Job.
   *
   * @param Jobs $job
   * @return Stage
   */
  public static function getSTAGE( Jobs $job ) {
    switch ( $job->etapa_atual ) {
      case 'A':
        return new StageA( $job );
      case 'B':
        return new StageB( $job );
      case 'C':
        return new StageC( $job );
      case 'D':
        return new StageD( $job );
      case 'E':
        return new StageE( $job );
      case 'F':
        return new StageF( $job );
      case 'G':
        return new StageG( $job );
      case 'Z':
        return new StageZ( $job );
      case 'L':
        return new StageL( $job );
      default:
        die( 'Erro: StageUtil::getSTAGE -> este etapa nao existe' );
    }
  }

  /**
   * Avança o Job para o etapa especificado.
   *
   * @param Jobs $job
   * @param string $etapa
   */
  public static function NEXT( Jobs $job, $etapa ) {
    // atualiza o atual
    $antigoStage = $job->StageTop();
    $antigoStage->update( [
      'saida' => Carbon::now()->format( 'Y-m-d H:i:s' )
    ] );

    // cria o novo etapa
    /** @var Stages $nextStage */
    $nextStage = $job->Stages()->create( [
      'etapa'   => $etapa,
      'entrada' => Carbon::now()->format( 'Y-m-d H:i:s' ),
      'salvo'   => null,
      'saida'   => null
    ] );

    // indica o novo etapa atual
    $job->update( [
      'etapa_atual' => $nextStage->etapa
    ] );
  }

  /**
   * Retrocedo uma etapa do job.
   *
   * @param Jobs $job
   */
  public static function BACK( Jobs $job ) {
    // atual
    $atualStage = $job->StageTop();
    $atualStage->delete();

    //  anterior
    $anteriorStage = $job->StageTop();
    $anteriorStage->update( [
      'salvo' => $anteriorStage->saida,
      'saida' => null
    ] );

    // Atualiza o atual no job
    $job->update( [
      'etapa_atual' => $anteriorStage->etapa
    ] );
  }

  /**
   * Armazena as Informacoes Basicas do Job.
   *
   * @param Jobs $job
   * @param JobRequest $request
   */
  public static function SALVAR_INFO( Jobs $job, JobRequest $request ) {
    $previsao_entrega = sprintf( '%s %s', $request->getOrNull( 'info_previsao_entrega_date' ), $request->getOrNull( 'info_previsao_entrega_time' ) );
    if ( trim( $previsao_entrega ) == '' ) {
      $previsao_entrega = null;
    } else {
      $previsao_entrega .= ':00';
    }

    $job->update( [
      'nome_job'          => $request->getOrNull( 'info_nome_job' ),
      'campanha_id'       => $request->getOrNull( 'info_campanha_id' ),
      'cliente_unidd_id'  => $request->getOrNull( 'info_cliente_unidd_id' ),
      'cliente_cdc_id'    => $request->getOrNull( 'info_cliente_cdc_id' ),
      'cod_receita'       => $request->getOrNull( 'info_cod_receita' ),
      'descricao_evento'  => $request->getOrNull( 'info_descricao_evento' ),
      'previsao_proposta' => $request->getOrNull( 'info_previsao_proposta' ),
      'previsao_entrega'  => $previsao_entrega
    ] );
  }

  /**
   * Salva os Briefings
   *
   * @param Jobs $job
   * @param JobRequest $request
   * @internal param $dados
   */
  public static function SALVAR_BRIEFING( Jobs $job, JobRequest $request ) {
    // Briefing Livre
    $job->BriefingLivre()->updateOrCreate( [], [
      'texto' => $request->getOrNull( 'briefing_livre' )
    ] );

    // Briefing Grafico
    $job->BriefingGrafico()->updateOrCreate( [], [
      // Medida
      'medida_aberto_altura'   => $request->getOrNull( 'bfgraf_medida_aberto_altura' ),
      'medida_aberto_largura'  => $request->getOrNull( 'bfgraf_medida_aberto_largura' ),
      'medida_fechado_altura'  => $request->getOrNull( 'bfgraf_medida_fechado_altura' ),
      'medida_fechado_largura' => $request->getOrNull( 'bfgraf_medida_fechado_largura' ),

      // Material
      'material_tipo'          => $request->getOrNull( 'bfgraf_material_tipo' ),
      'material_cores'         => $request->getOrNull( 'bfgraf_material_cores' ),
      'material_cores_escala'  => $request->getOrNull( 'bfgraf_material_cores_escala' ),

      // Quantidade de Páginas/Lâminas
      'paginas'                => $request->getOrNull( 'bfgraf_paginas' ),
      'laminas'                => $request->getOrNull( 'bfgraf_laminas' ),

      // Acabamento
      'acabamento'             => $request->getOrNull( 'bfgraf_acabamento' ),
      'faca'                   => $request->getOrNull( 'bfgraf_faca' ),

      // Tiragem
      'tiragem'                => $request->getOrNull( 'bfgraf_tiragem' ),
      'tiragem_descricao'      => $request->getOrNull( 'bfgraf_tiragem_descricao' ),

      // Entrega
      'entrega_data'           => $request->getOrNull( 'bfgraf_entrega_data' ),
      'entrega_hora'           => $request->getOrNull( 'bfgraf_entrega_hora' ),
      'entrega_endereco'       => $request->getOrNull( 'bfgraf_entrega_endereco' ),
      'entrega_telefone'       => $request->getOrNull( 'bfgraf_entrega_telefone' ),
      'entrega_responsavel'    => $request->getOrNull( 'bfgraf_entrega_responsavel' ),

      //  DADOS PARA FATURA ?
      //  ENDEREÇO PARA ENTREGA DE FATURA ?
      //  LOCAL ONDE COTAR (PREENCHIMENTO MARIS OU JULIANA) ?

      'obs' => $request->getOrNull( 'bfgraf_obs' ),
    ] );

    // Briefing Web
    $job->BriefingWeb()->updateOrCreate( [], [
      // Resumo
      'descricao'              => $request->getOrNull( 'bfweb_descricao' ),

      // arquitetura de navegacao
      'arquitetura'            => $request->getOrNull( 'bfweb_arquitetura' ),

      // go live
      'data_online'            => $request->getOrNull( 'bfweb_data_online' ),

      // Quais são as características do servidor?
      'server_web'             => $request->getOrNull( 'bfweb_server_web' ),
      'server_db'              => $request->getOrNull( 'bfweb_server_db' ),
      'server_linguagem'       => $request->getOrNull( 'bfweb_server_linguagem' ),
      'server_so'              => $request->getOrNull( 'bfweb_server_so' ),
      'server_navegador'       => $request->getOrNull( 'bfweb_server_navegador' ),
      'server_dominio'         => $request->getOrNull( 'bfweb_server_dominio' ),

      // Quais são as características do servidor do cliente?
      'server_tamanho'         => $request->getOrNull( 'bfweb_server_tamanho' ),
      'server_trafego'         => $request->getOrNull( 'bfweb_server_trafego' ),
      'server_disponibilidade' => $request->getOrNull( 'bfweb_server_disponibilidade' ),
      'server_bd'              => $request->getOrNull( 'bfweb_server_bd' ), // enum  (mesma instância ou instância separada?)

      // A Cadaris deverá efetuar a compra do domínio?
      'dominio_principal'      => $request->getOrNull( 'bfweb_dominio_principal' ),
      'dominio_secundario1'    => $request->getOrNull( 'bfweb_dominio_secundario1' ),
      'dominio_secundario2'    => $request->getOrNull( 'bfweb_dominio_secundario2' ),

      // Tipo do Trabalho
      'tipo_site'              => $request->getOrNull( 'bfweb_tipo_site', 0 ),
      'tipo_portal'            => $request->getOrNull( 'bfweb_tipo_portal', 0 ),
      'tipo_app_web'           => $request->getOrNull( 'bfweb_tipo_app_web', 0 ),
      'tipo_app_ios'           => $request->getOrNull( 'bfweb_tipo_app_ios', 0 ),
      'tipo_app_android'       => $request->getOrNull( 'bfweb_tipo_app_android', 0 ),
      'tipo_app_windows_phone' => $request->getOrNull( 'bfweb_tipo_app_windows_phone', 0 ),


      // Tecnologia
      // Front-End
      'tec_front_bootstrap'    => $request->getOrNull( 'bfweb_tec_front_bootstrap', 0 ),
      'tec_front_html5'        => $request->getOrNull( 'bfweb_tec_front_html5', 0 ),
      'tec_front_css'          => $request->getOrNull( 'bfweb_tec_front_css', 0 ),
      'tec_front_outra'        => $request->getOrNull( 'bfweb_tec_front_outra' ),

      // CMS
      'tec_cms_wordpress'      => $request->getOrNull( 'bfweb_tec_cms_wordpress', 0 ),
      'tec_cms_outra'          => $request->getOrNull( 'bfweb_tec_cms_outra' ),

      // Bibliotecas Javascript
      'tec_js_jquery'          => $request->getOrNull( 'bfweb_tec_js_jquery', 0 ),
      'tec_js_angularjs'       => $request->getOrNull( 'bfweb_tec_js_angularjs', 0 ),
      'tec_js_createjs'        => $request->getOrNull( 'bfweb_tec_js_createjs', 0 ),
      'tec_js_outra'           => $request->getOrNull( 'bfweb_tec_js_outra' ),

      // Linguagem
      'tec_linguagem_php'      => $request->getOrNull( 'bfweb_tec_linguagem_php', 0 ),
      'tec_linguagem_outra'    => $request->getOrNull( 'bfweb_tec_linguagem_outra' ),

      // Banco de dados
      'tec_bd_mysql'           => $request->getOrNull( 'bfweb_tec_bd_mysql', 0 ),
      'tec_bd_sqlserver'       => $request->getOrNull( 'bfweb_tec_bd_sqlserver', 0 ),
      'tec_bd_outra'           => $request->getOrNull( 'bfweb_tec_bd_outra' ),

      // Plugins
      'tec_plugin_ga'          => $request->getOrNull( 'bfweb_tec_plugin_ga', 0 ),
      'tec_plugin_sm'          => $request->getOrNull( 'bfweb_tec_plugin_sm', 0 ),
      'tec_plugin_metatag'     => $request->getOrNull( 'bfweb_tec_plugin_metatag', 0 ),
      'tec_plugin_outra'       => $request->getOrNull( 'bfweb_tec_plugin_outra' ),

      // Servidor
      'tec_os_linux'           => $request->getOrNull( 'bfweb_tec_os_linux', 0 ),
      'tec_os_windows'         => $request->getOrNull( 'bfweb_tec_os_windows', 0 ),
      'tec_os_outra'           => $request->getOrNull( 'bfweb_tec_os_outra' ),

      // Mobile
      'tec_mobile_phonegap'    => $request->getOrNull( 'bfweb_tec_mobile_phonegap', 0 ),
      'tec_mobile_cordova'     => $request->getOrNull( 'bfweb_tec_mobile_cordova', 0 ),
      'tec_mobile_outra'       => $request->getOrNull( 'bfweb_tec_mobile_outra' ),

      'obs' => $request->getOrNull( 'bfweb_obs' ),

    ] );

    // Briefing Brinde
    $job->BriefingBrinde()->updateOrCreate( [], [
      'nome_material'           => $request->getOrNull( 'bfbrinde_nome_material' ),
      'resumo'                  => $request->getOrNull( 'bfbrinde_resumo' ),
      'formato_aberto_altura'   => $request->getOrNull( 'bfbrinde_formato_aberto_altura' ),
      'formato_aberto_largura'  => $request->getOrNull( 'bfbrinde_formato_aberto_largura' ),
      'formato_fechado_altura'  => $request->getOrNull( 'bfbrinde_formato_fechado_altura' ),
      'formato_fechado_largura' => $request->getOrNull( 'bfbrinde_formato_fechado_largura' ),
      'tiragem'                 => $request->getOrNull( 'bfbrinde_tiragem' ),
      'tiragem_descricao'       => $request->getOrNull( 'bfbrinde_tiragem_descricao' ),
      'material_tipo'           => $request->getOrNull( 'bfbrinde_material_tipo' ),
      'material_cores'          => $request->getOrNull( 'bfbrinde_material_cores' ),
      'material_cores_escala'   => $request->getOrNull( 'bfbrinde_material_cores_escala' ),
      'tipo_gravacao'           => $request->getOrNull( 'bfbrinde_tipo_gravacao' ),
      'referencias'             => $request->getOrNull( 'bfbrinde_referencias' ),
      'entrega_data'            => $request->getOrNull( 'bfbrinde_entrega_data' ),
      'entrega_hora'            => $request->getOrNull( 'bfbrinde_entrega_hora' ),
      'entrega_endereco'        => $request->getOrNull( 'bfbrinde_entrega_endereco' ),
      'entrega_telefone'        => $request->getOrNull( 'bfbrinde_entrega_telefone' ),
      'entrega_responsavel'     => $request->getOrNull( 'bfbrinde_entrega_responsavel' ),
      'obs'                     => $request->getOrNull( 'bfbrinde_obs' ),
    ] );
  }

  /**
   * Salva o Cronograma.
   *
   * @param Jobs $job
   * @param JobRequest $request
   * @internal param $dados
   */
  public static function SALVAR_CRONOGRAMA( Jobs $job, JobRequest $request ) {
    $job->Cronograma()->updateOrCreate( [], [
      'json' => $request->getOrNull( 'cronograma_hidden', '[]' )
    ] );
  }

  /**
   * Armazena um Log deste Job.
   *
   * @param Jobs $job
   * @param string $etapa
   * @param boolean $tipo
   */
  public static function LOG( Jobs $job, $etapa, $tipo ) {
    $funcionarios_id = Auth::user()->funcionarios_id;
    /** @var Funcionarios $func */
    $func = Funcionarios::find( $funcionarios_id );
    if ( $func ) {
      $func_nome = $func->nome;
    } else {
      $func_nome = 'Usuário ' . Auth::user()->name;
    }

    $job->Logs()->create( [
      'data'             => Carbon::now()->toDateTimeString(),
      'etapa'            => $etapa,
      'tipo'             => $tipo,
      'nome_funcionario' => $func_nome,
    ] );
  }

}