<?php

namespace App\Http\Controllers\Financeiro;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Models\Financeiro\ContaDeConsumo;
use App\Models\Financeiro\ContaDeConsumoGerador;
use Illuminate\Http\Request;

class ContaDeConsumoController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Financeiro ) );
  }

  /**
   * Lista ContaDeConsumo.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\View\View
   */
  public function index() {
    $indexSearch = ContaDeConsumo::IndexSearchMap();
    $consumos    = ContaDeConsumo::listarPaginadoBuscavelOrdenavel( ContaDeConsumo::query(), $indexSearch );
    $caixaVoltar = \Request::has( 'caixa' ) ? route( 'financeiro.caixa.show_periodo', \Request::get( 'caixa' ) ) : '';

    return view( 'Financeiro.ContasConsumos.index', compact( 'consumos', 'indexSearch', 'caixaVoltar' ) );
  }

  /**
   * Exibem o formulario para criar uma nova ContaDeConsumo.
   *
   * @return \Illuminate\Http\Response
   */
  public function create() {
    $gerador = null;
    $ge      = intval( \Request::get( 'gerador' ) );
    $gerador = ContaDeConsumoGerador::find( $ge );
    if ( $gerador ) {
      $gerador = $gerador->toArray();
    }

    return view( 'Financeiro.ContasConsumos.create', compact( 'gerador' ) );
  }

  /**
   * Armazena uma nova ContaDeConsumo no BD.
   *
   * @param \Illuminate\Http\Request $request
   * @return \Illuminate\Http\Response
   */
  public function store( Request $request ) {
    /** @var ContaDeConsumo $consumo */
    $consumo = ContaDeConsumo::create( $request->all() );

    if ( $request->hasFile( 'file_boleto' ) ) {
      $fileB     = $request->file( 'file_boleto' );
      $filenameB = $fileB->getClientOriginalName();
      $localB    = $consumo->getLocalBoleto();
      $fileB->move( $localB, $filenameB );

      $consumo->boleto = $filenameB;
      $consumo->save();
    }
    if ( $request->hasFile( 'file_comprovante' ) ) {
      $fileC     = $request->file( 'file_comprovante' );
      $filenameC = $fileC->getClientOriginalName();
      $localC    = $consumo->getLocalComprovante();
      $fileC->move( $localC, $filenameC );

      $consumo->comprovante = $filenameC;
      $consumo->save();
    }
    $consumo->reSyncStore();

    flash( 'Conta de Consumo criada com sucesso!', 'success' );

    return redirect()->route( 'financeiro.contasdeconsumo.index' );
  }


  /**
   * Exibe o formulario para editar uma ContaDeConsumo no BD.
   *
   * @param int $id
   * @return \Illuminate\Http\Response
   */
  public function edit( $id ) {
    $consumo = ContaDeConsumo::findOrFail( $id );
    $gerador = null;

    $periodo = \Request::get( 'caixa', '' );
    if ( $periodo ) {
      $caixaVoltar = route( 'financeiro.caixa.show_periodo', $periodo );
    } else {
      $caixaVoltar = route( 'financeiro.contasdeconsumo.index' );
    }

    return view( 'Financeiro.ContasConsumos.edit', compact( 'consumo', 'gerador', 'periodo', 'caixaVoltar' ) );
  }

  /**
   * Atualiza uma ContaDeConsumo no BD.
   *
   * @param \Illuminate\Http\Request $request
   * @param int $id
   * @return \Illuminate\Http\Response
   */
  public function update( Request $request, $id ) {
    /** @var ContaDeConsumo $consumo */
    $consumo = ContaDeConsumo::findOrFail( $id );
    $consumo->reSyncUpdateBegin();
    $consumo->update( $request->all() );

    $reSave = false;
    if ( $request->has( 'deletar_boleto' ) && $consumo->boleto ) {
      $localB = public_path( $consumo->getLocalBoletoLink() );
      if ( file_exists( $localB ) ) {
        unlink( $localB );
      }
      $consumo->boleto = null;

      $reSave = true;
    }
    if ( $request->hasFile( 'file_boleto' ) ) {
      $localB = public_path( $consumo->getLocalBoletoLink() );
      if ( file_exists( $localB ) && $consumo->boleto ) {
        unlink( $localB );
      }
      $consumo->boleto = null;

      $fileB     = $request->file( 'file_boleto' );
      $filenameB = $fileB->getClientOriginalName();
      $localB    = $consumo->getLocalBoleto();
      $fileB->move( $localB, $filenameB );

      $consumo->boleto = $filenameB;

      $reSave = true;
    }

    if ( $request->has( 'deletar_comprovante' ) && $consumo->comprovante ) {
      $localC = public_path( $consumo->getLocalComprovanteLink() );
      if ( file_exists( $localC ) ) {
        unlink( $localC );
      }
      $consumo->comprovante = null;

      $reSave = true;
    }
    if ( $request->hasFile( 'file_comprovante' ) ) {
      $localC = public_path( $consumo->getLocalComprovanteLink() );
      if ( file_exists( $localC ) && $consumo->comprovante ) {
        unlink( $localC );
      }
      $consumo->comprovante = null;

      $fileC     = $request->file( 'file_comprovante' );
      $filenameC = $fileC->getClientOriginalName();
      $localC    = $consumo->getLocalComprovante();
      $fileC->move( $localC, $filenameC );

      $consumo->comprovante = $filenameC;

      $reSave = true;
    }
    if ( $reSave ) {
      $consumo->save();
    }
    $consumo->reSyncUpdateEnd();

    flash( 'Conta de Consumo atualizada com sucesso!', 'success' );

    $periodo = \Request::get( '_voltar', '' );
    if ( $periodo ) {
      return redirect()->route( 'financeiro.caixa.show_periodo', $periodo );
    }

    return redirect()->route( 'financeiro.contasdeconsumo.index' );
  }

  /**
   * Remove uma ContaDeConsumo no BD.
   *
   * @param int $id
   * @return \Illuminate\Http\Response
   */
  public function destroy( $id ) {
    /** @var ContaDeConsumo $consumo */
    $consumo = ContaDeConsumo::findOrFail( $id );

    if ( $consumo->boleto ) {
      $localB = public_path( $consumo->getLocalBoletoLink() );
      if ( file_exists( $localB ) ) {
        unlink( $localB );
      }
    }
    if ( $consumo->comprovante ) {
      $localC = public_path( $consumo->getLocalComprovanteLink() );
      if ( file_exists( $localC ) ) {
        unlink( $localC );
      }
    }
    $consumo->reSyncDestroy();
    $consumo->delete();

    flash( 'Conta de Consumo deletada com sucesso!', 'success' );

    return redirect()->route( 'financeiro.contasdeconsumo.index' );
  }
}
