<?php

namespace App\Http\Controllers\Financeiro;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Models\Financeiro\ContaFornecedor;
use Illuminate\Http\Request;

class ContaFornecedorController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Financeiro ) );
  }

  /**
   * Lista as ContaFornecedor.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\View\View
   */
  public function index() {
    $indexSearch  = ContaFornecedor::IndexSearchMap();
    $fornecedores = ContaFornecedor::listarPaginadoBuscavelOrdenavel( ContaFornecedor::query(), $indexSearch );
    $caixaVoltar  = \Request::has( 'caixa' ) ? route( 'financeiro.caixa.show_periodo', \Request::get( 'caixa' ) ) : '';

    return view( 'Financeiro.ContasFornecedores.index', compact( 'fornecedores', 'indexSearch', 'caixaVoltar' ) );
  }

  /**
   * Exibem o formulario para criar uma nova ContaFornecedor.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\View\View
   */
  public function create() {

    return view( 'Financeiro.ContasFornecedores.create' );
  }

  /**
   * Armazena uma nova ContaFornecedor no BD.
   *
   * @param \Illuminate\Http\Request $request
   * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response
   */
  public function store( Request $request ) {
    /** @var ContaFornecedor $fornecedor */
    $fornecedor = ContaFornecedor::create( $request->all() );

    if ( $request->hasFile( 'file_boleto' ) ) {
      $fileB     = $request->file( 'file_boleto' );
      $filenameB = $fileB->getClientOriginalName();
      $localB    = $fornecedor->getLocalBoleto();
      $fileB->move( $localB, $filenameB );

      $fornecedor->boleto = $filenameB;
      $fornecedor->save();
    }
    if ( $request->hasFile( 'file_comprovante' ) ) {
      $fileC     = $request->file( 'file_comprovante' );
      $filenameC = $fileC->getClientOriginalName();
      $localC    = $fornecedor->getLocalComprovante();
      $fileC->move( $localC, $filenameC );

      $fornecedor->comprovante = $filenameC;
      $fornecedor->save();
    }
    $fornecedor->reSyncStore();

    flash( 'Conta de Fornecedor criada com sucesso!', 'success' );

    return redirect()->route( 'financeiro.fornecedores.index' );
  }

  /**
   * Exibe o formulario para editar uma ContaFornecedor no BD.
   *
   * @param int $id
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\View\View
   */
  public function edit( $id ) {
    $fornecedor = ContaFornecedor::findOrFail( $id );

    $periodo = \Request::get( 'caixa', '' );
    if ( $periodo ) {
      $caixaVoltar = route( 'financeiro.caixa.show_periodo', $periodo );
    } else {
      $caixaVoltar = route( 'financeiro.fornecedores.index' );
    }

    return view( 'Financeiro.ContasFornecedores.edit', compact( 'fornecedor', 'periodo', 'caixaVoltar' ) );
  }

  /**
   * Atualiza uma ContaFornecedor no BD.
   *
   * @param \Illuminate\Http\Request $request
   * @param int $id
   * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response
   */
  public function update( Request $request, $id ) {
    /** @var ContaFornecedor $fornecedor */
    $fornecedor = ContaFornecedor::findOrFail( $id );
    $fornecedor->reSyncUpdateBegin();
    $fornecedor->update( $request->all() );

    $reSave = false;
    if ( $request->has( 'deletar_boleto' ) && $fornecedor->boleto ) {
      $localB = public_path( $fornecedor->getLocalBoletoLink() );
      if ( file_exists( $localB ) ) {
        unlink( $localB );
      }
      $fornecedor->boleto = null;

      $reSave = true;
    }
    if ( $request->hasFile( 'file_boleto' ) ) {
      $localB = public_path( $fornecedor->getLocalBoletoLink() );
      if ( file_exists( $localB ) && $fornecedor->boleto ) {
        unlink( $localB );
      }
      $fornecedor->boleto = null;

      $fileB     = $request->file( 'file_boleto' );
      $filenameB = $fileB->getClientOriginalName();
      $localB    = $fornecedor->getLocalBoleto();
      $fileB->move( $localB, $filenameB );

      $fornecedor->boleto = $filenameB;

      $reSave = true;
    }

    if ( $request->has( 'deletar_comprovante' ) && $fornecedor->comprovante ) {
      $localC = public_path( $fornecedor->getLocalComprovanteLink() );
      if ( file_exists( $localC ) ) {
        unlink( $localC );
      }
      $fornecedor->comprovante = null;

      $reSave = true;
    }
    if ( $request->hasFile( 'file_comprovante' ) ) {
      $localC = public_path( $fornecedor->getLocalComprovanteLink() );
      if ( file_exists( $localC ) && $fornecedor->comprovante ) {
        unlink( $localC );
      }
      $fornecedor->comprovante = null;

      $fileC     = $request->file( 'file_comprovante' );
      $filenameC = $fileC->getClientOriginalName();
      $localC    = $fornecedor->getLocalComprovante();
      $fileC->move( $localC, $filenameC );

      $fornecedor->comprovante = $filenameC;

      $reSave = true;
    }
    if ( $reSave ) {
      $fornecedor->save();
    }
    $fornecedor->reSyncUpdateEnd();

    flash( 'Conta de Fornecedor atualizada com sucesso!', 'success' );

    $periodo = \Request::get( '_voltar', '' );
    if ( $periodo ) {
      return redirect()->route( 'financeiro.caixa.show_periodo', $periodo );
    }

    return redirect()->route( 'financeiro.fornecedores.index' );
  }

  /**
   * Remove uma ContaFornecedor no BD.
   *
   * @param int $id
   * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response
   */
  public function destroy( $id ) {
    /** @var ContaFornecedor $fornecedor */
    $fornecedor = ContaFornecedor::findOrFail( $id );

    if ( $fornecedor->boleto ) {
      $localB = public_path( $fornecedor->getLocalBoletoLink() );
      if ( file_exists( $localB ) ) {
        unlink( $localB );
      }
    }
    if ( $fornecedor->comprovante ) {
      $localC = public_path( $fornecedor->getLocalComprovanteLink() );
      if ( file_exists( $localC ) ) {
        unlink( $localC );
      }
    }
    $fornecedor->reSyncDestroy();
    $fornecedor->delete();

    flash( 'Conta de Fornecedor deletada com sucesso!', 'success' );

    return redirect()->route( 'financeiro.fornecedores.index' );
  }
}
