<?php

namespace App\Http\Controllers\Financeiro;

use App\CRP;
use App\Http\Controllers\Controller;
use App\Models\Financeiro\OutroRecebimento;
use Illuminate\Http\Request;

class OutroRecebimentoController extends Controller {
  /**
   * Cria uma nova instancia deste Controller.
   *
   */
  public function __construct() {
    $this->middleware( 'auth' );
    $this->middleware( CRP::role( CRP::P_Financeiro ) );
  }

  /**
   * Lista as OutroRecebimento.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\View\View
   */
  public function index() {
    $indexSearch  = OutroRecebimento::IndexSearchMap();
    $recebimentos = OutroRecebimento::listarPaginadoBuscavelOrdenavel( OutroRecebimento::query(), $indexSearch );
    $caixaVoltar  = \Request::has( 'caixa' ) ? route( 'financeiro.caixa.show_periodo', \Request::get( 'caixa' ) ) : '';

    return view( 'Financeiro.OutrosRecebimentos.index', compact( 'recebimentos', 'indexSearch', 'caixaVoltar' ) );
  }

  /**
   * Exibem o formulario para criar uma nova OutroRecebimento.
   *
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\View\View
   */
  public function create() {
    return view( 'Financeiro.OutrosRecebimentos.create' );
  }

  /**
   * Armazena uma nova OutroRecebimento no BD.
   *
   * @param \Illuminate\Http\Request $request
   * @return \Illuminate\Http\RedirectResponse
   */
  public function store( Request $request ) {
    /** @var OutroRecebimento $recebimento */
    $recebimento = OutroRecebimento::create( $request->all() );

    if ( $request->hasFile( 'file_boleto' ) ) {
      $fileB     = $request->file( 'file_boleto' );
      $filenameB = $fileB->getClientOriginalName();
      $localB    = $recebimento->getLocalBoleto();
      $fileB->move( $localB, $filenameB );

      $recebimento->boleto = $filenameB;
      $recebimento->save();
    }
    if ( $request->hasFile( 'file_comprovante' ) ) {
      $fileC     = $request->file( 'file_comprovante' );
      $filenameC = $fileC->getClientOriginalName();
      $localC    = $recebimento->getLocalComprovante();
      $fileC->move( $localC, $filenameC );

      $recebimento->comprovante = $filenameC;
      $recebimento->save();
    }
    $recebimento->reSyncStore();

    flash( 'Recebimento criado com sucesso!', 'success' );

    return redirect()->route( 'financeiro.recebimentos.index' );
  }

  /**
   * Exibe o formulario para editar uma OutroRecebimento no BD.
   *
   * @param int $id
   * @return \Illuminate\Contracts\View\Factory|\Illuminate\Foundation\Application|\Illuminate\Http\Response|\Illuminate\View\View
   */
  public function edit( $id ) {
    $recebimento = OutroRecebimento::findOrFail( $id );

    $periodo = \Request::get( 'caixa', '' );
    if ( $periodo ) {
      $caixaVoltar = route( 'financeiro.caixa.show_periodo', $periodo );
    } else {
      $caixaVoltar = route( 'financeiro.recebimentos.index' );
    }

    return view( 'Financeiro.OutrosRecebimentos.edit', compact( 'recebimento', 'periodo', 'caixaVoltar' ) );
  }

  /**
   * Atualiza uma OutroRecebimento no BD.
   *
   * @param \Illuminate\Http\Request $request
   * @param int $id
   * @return \Illuminate\Http\RedirectResponse
   */
  public function update( Request $request, $id ) {
    /** @var OutroRecebimento $recebimento */
    $recebimento = OutroRecebimento::findOrFail( $id );
    $recebimento->reSyncUpdateBegin();
    $recebimento->update( $request->all() );

    $reSave = false;
    if ( $request->has( 'deletar_boleto' ) && $recebimento->boleto ) {
      $localB = public_path( $recebimento->getLocalBoletoLink() );
      if ( file_exists( $localB ) ) {
        unlink( $localB );
      }
      $recebimento->boleto = null;

      $reSave = true;
    }
    if ( $request->hasFile( 'file_boleto' ) ) {
      $localB = public_path( $recebimento->getLocalBoletoLink() );
      if ( file_exists( $localB ) && $recebimento->boleto ) {
        unlink( $localB );
      }
      $recebimento->boleto = null;

      $fileB     = $request->file( 'file_boleto' );
      $filenameB = $fileB->getClientOriginalName();
      $localB    = $recebimento->getLocalBoleto();
      $fileB->move( $localB, $filenameB );

      $recebimento->boleto = $filenameB;

      $reSave = true;
    }

    if ( $request->has( 'deletar_comprovante' ) && $recebimento->comprovante ) {
      $localC = public_path( $recebimento->getLocalComprovanteLink() );
      if ( file_exists( $localC ) ) {
        unlink( $localC );
      }
      $recebimento->comprovante = null;

      $reSave = true;
    }
    if ( $request->hasFile( 'file_comprovante' ) ) {
      $localC = public_path( $recebimento->getLocalComprovanteLink() );
      if ( file_exists( $localC ) && $recebimento->comprovante ) {
        unlink( $localC );
      }
      $recebimento->comprovante = null;

      $fileC     = $request->file( 'file_comprovante' );
      $filenameC = $fileC->getClientOriginalName();
      $localC    = $recebimento->getLocalComprovante();
      $fileC->move( $localC, $filenameC );

      $recebimento->comprovante = $filenameC;

      $reSave = true;
    }
    if ( $reSave ) {
      $recebimento->save();
    }
    $recebimento->reSyncUpdateEnd();

    flash( 'Recebimento atualizado com sucesso!', 'success' );

    $periodo = \Request::get( '_voltar', '' );
    if ( $periodo ) {
      return redirect()->route( 'financeiro.caixa.show_periodo', $periodo );
    }

    return redirect()->route( 'financeiro.recebimentos.index' );
  }

  /**
   * Remove uma OutroRecebimento no BD.
   *
   * @param int $id
   * @return \Illuminate\Http\RedirectResponse
   * @throws \Exception
   */
  public function destroy( $id ) {
    /** @var OutroRecebimento $recebimento */
    $recebimento = OutroRecebimento::findOrFail( $id );

    if ( $recebimento->boleto ) {
      $localB = public_path( $recebimento->getLocalBoletoLink() );
      if ( file_exists( $localB ) ) {
        unlink( $localB );
      }
    }
    if ( $recebimento->comprovante ) {
      $localC = public_path( $recebimento->getLocalComprovanteLink() );
      if ( file_exists( $localC ) ) {
        unlink( $localC );
      }
    }
    $recebimento->reSyncDestroy();
    $recebimento->delete();

    flash( 'Recebimento deletado com sucesso!', 'success' );

    return redirect()->route( 'financeiro.recebimentos.index' );
  }
}
