<?php

namespace App\Models\Financeiro;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;

/**
 * App\Models\Financeiro\ContaBancaria
 *
 * @property int $id
 * @property string $nome
 * @property int $banco
 * @property string $agencia
 * @property string $conta
 * @property string $data_inicial
 * @property int $valor_inicial
 * @property \Carbon\Carbon $created_at
 * @property \Carbon\Carbon $updated_at
 * @property-read \Illuminate\Database\Eloquent\Collection|\App\Models\Financeiro\ContaBancariaSync[] $ContaBancariaSync
 * @property-read \Illuminate\Database\Eloquent\Collection|\App\Models\Financeiro\ContaFornecedor[] $ContasFornecedores
 * @property-read \Illuminate\Database\Eloquent\Collection|\App\Models\Financeiro\JobParcela[] $JobParcelas
 * @property-read \Illuminate\Database\Eloquent\Collection|\App\Models\Financeiro\Transferencia[] $TransferenciasEntradas
 * @property-read \Illuminate\Database\Eloquent\Collection|\App\Models\Financeiro\Transferencia[] $TransferenciasSaidas
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereAgencia( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereBanco( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereConta( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereCreatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereDataInicial( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereNome( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereUpdatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\ContaBancaria whereValorInicial( $value )
 * @mixin \Eloquent
 */
class ContaBancaria extends Model {
  /**
   * A tabela associada com este Model.
   *
   * @var string
   */
  protected $table = 'financeiro_contas_bancarias';

  /**
   * Os atributos que são atribuíveis em massa.
   *
   * @var array
   */
  protected $fillable = [
    'nome',
    'banco',
    'agencia',
    'conta',
    'data_inicial',
    'valor_inicial'
  ];

  /**
   * Enumerador de Contas Bancarias.
   *
   * @return array
   */
  public static function enum_contas() {
    $contas = [];
    foreach ( ContaBancaria::all() as $conta ) {
      $contas[ $conta->id ] = $conta->nome;
    }

    return $contas;
  }

  /**
   * Enumerador de Contas Bancarias com a opcao Pago pelo Cliente.
   *
   * @return array
   */
  public static function enum_contas_pelo_cliente() {
    $contas        = [];
    $contas[ '0' ] = '-- Pago pelo Cliente --';
    foreach ( ContaBancaria::all() as $conta ) {
      $contas[ $conta->id ] = $conta->nome;
    }

    return $contas;
  }

  /**
   * Retorna as Parcela de Job desta Conta Bancaria.
   *
   * @return \Illuminate\Database\Eloquent\Relations\HasMany | \Illuminate\Database\Query\Builder
   */
  public function JobParcelas() {
    return $this->hasMany( JobParcela::class, 'conta_id' );
  }

  /**
   * Retorna a Transferencia de Saida associada a esta Conta Bancaria.
   *
   * @return \Illuminate\Database\Eloquent\Relations\HasMany | \Illuminate\Database\Query\Builder
   */
  public function TransferenciasSaidas() {
    return $this->hasMany( Transferencia::class, 'conta_saida_id' );
  }

  /**
   * Retorna a Transferencia de Entrada associada a esta Conta Bancaria.
   *
   * @return \Illuminate\Database\Eloquent\Relations\HasMany
   */
  public function TransferenciasEntradas() {
    return $this->hasMany( Transferencia::class, 'conta_entrada_id' );
  }

  /**
   * Retorna as Contas de Fornecedores associada a esta Conta Bancaria.
   *
   * @return \Illuminate\Database\Eloquent\Relations\HasMany | \Illuminate\Database\Query\Builder
   */
  public function ContasFornecedores() {
    return $this->hasMany( ContaFornecedor::class, 'conta_id' );
  }

  /**
   * Retorna as ContaBancariaSync associada a esta Conta Bancaria.
   *
   * @return \Illuminate\Database\Eloquent\Relations\HasMany | \Illuminate\Database\Query\Builder
   */
  public function ContaBancariaSync() {
    return $this->hasMany( ContaBancariaSync::class, 'conta_id' );
  }

  /**
   * @param string|\Carbon\Carbon $target
   * @return integer
   */
  public function CalcularValorInicial( $target ) {
    $target     = Carbon::parse( $target );
    $dataInicio = $target->format( 'Y-m-d' );

    $count = $this->ContaBancariaSync()->count();
    /** @var ContaBancariaSync $ultimo */
    $ultimo = null;
    if ( $count == 0 ) {
      // se nao tem nenhum, cria!
      $dia1   = Carbon::parse( $this->data_inicial )->startOfMonth();
      $ultimo = $this->ContaBancariaSync()->create( [
        'data'  => $dia1->format( 'Y-m-d' ),
        'valor' => $this->valor_inicial,
      ] );
    } else {
      $ultimo = $this->ContaBancariaSync()
                     ->where( 'data', '<=', $dataInicio )
                     ->orderBy( 'data', 'desc' )
                     ->first();
    }
    if ( !$ultimo ) {

      return 0;
    }
    if ( $ultimo->data > $target ) {

      return 0;
    }

    // sync meses anteriores até chegar no ultimo
    while ( $ultimo->data->format( 'Y-m' ) < $target->format( 'Y-m' ) ) {
      // mes atual
      $mesAtual = $ultimo->data->copy();
      $inicio   = $mesAtual->copy()->startOfMonth();
      $fim      = $mesAtual->copy()->endOfMonth();

      // total deste mes
      $valorMes = $this->getValorTotalPeriodo( $inicio, $fim );

      // prox mes
      $proxMes = $mesAtual->copy()->addMonth()->startOfMonth();

      $ultimo = $this->ContaBancariaSync()->create( [
        'data'  => $proxMes->format( 'Y-m-d' ),
        'valor' => $ultimo->valor + $valorMes,
      ] );
    }

    // completa ate o dia
    $diaAntes = $target->copy()->subDay();
    $valorDia = $ultimo->valor + $this->getValorTotalPeriodo( $ultimo->data, $diaAntes );

    return $valorDia;
  }

  /**
   * Retorna o Valor Total dos eventos em um periodo.
   *
   * @param Carbon $dt_inicio
   * @param Carbon $dt_fim
   * @return integer
   */
  private function getValorTotalPeriodo( $dt_inicio, $dt_fim ) {
    $inicio = $dt_inicio->format( 'Y-m-d' );
    $fim    = $dt_fim->format( 'Y-m-d' );
    $VALOR  = 0;

    /// PAGAR
    // Contas Consumo
    $VALOR -= ContaDeConsumo
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( ContaDeConsumo $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, ContaDeConsumo $item ) { return $carry + $item->valor; }, 0 );

    // Pagamento de salario
    $VALOR -= Salario
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( Salario $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, Salario $item ) { return $carry + $item->valor_total; }, 0 );

    // Pagamento de Ferias
    $VALOR -= Ferias
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( Ferias $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, Ferias $item ) { return $carry + $item->valor_total_liquido; }, 0 );

    // Pagamento do Fornecedor pelo Job Realizado
    $VALOR -= ContaFornecedor
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( ContaFornecedor $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, ContaFornecedor $item ) { return $carry + $item->valor; }, 0 );

    // Transferencia Saida
    $VALOR -= Transferencia
      ::PeriodoSaida( $inicio, $fim, $this->id )
      ->reduce( function ( int $carry, Transferencia $item ) { return $carry + $item->valor + $item->valor_taxa; }, 0 );

    // Retirada de Sócios
    $VALOR -= RetiradaDeSocios
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( RetiradaDeSocios $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, RetiradaDeSocios $item ) { return $carry + $item->valor; }, 0 );

    // Rescisao
    $VALOR -= Rescisao
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( Rescisao $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, Rescisao $item ) { return $carry + $item->valor_total; }, 0 );

    // 13 Salario
    $VALOR -= Salario13
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( Salario13 $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, Salario13 $item ) { return $carry + $item->valor_total; }, 0 );

    /// RECEBER
    // Recebimento do Cliente pelo Job Realizado
    $VALOR += JobParcela
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( JobParcela $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, JobParcela $item ) { return $carry + $item->valor_liquido; }, 0 );

    // Transferência Entrada
    $VALOR += Transferencia
      ::PeriodoEntrada( $inicio, $fim, $this->id )
      ->reduce( function ( int $carry, Transferencia $item ) { return $carry + $item->valor; }, 0 );

    // Investimentos
    $VALOR += Investimentos
      ::Periodo( $inicio, $fim, $this->id )
      ->reduce( function ( int $carry, Investimentos $item ) { return $carry + $item->valor_liquido; }, 0 );

    // Outros Recebimentos
    $VALOR += OutroRecebimento
      ::Periodo( $inicio, $fim, $this->id )
      ->filter( function ( OutroRecebimento $item ) { return $item->pago === 1; } )
      ->reduce( function ( int $carry, OutroRecebimento $item ) { return $carry + $item->valor; }, 0 );

    return $VALOR;
  }


  /**
   * ReSync ContaBancariaSync desta Conta Bancaria.
   *
   * @param string $dataEvento
   */
  public function reSyncConta( $dataEvento ) {
    $dataEvento    = Carbon::parse( $dataEvento );
    $dataInicioAlt = $dataEvento->copy()->startOfMonth();
    $dataFimAlt    = $dataEvento->copy()->endOfMonth();

    /** @var ContaBancariaSync $ContaMesAlt */
    $ContaMesAlt = $this->ContaBancariaSync()
                        ->where( 'data', '=', $dataInicioAlt->format( 'Y-m-d' ) )
                        ->first();
    if ( !$ContaMesAlt ) {
      return;
    }

    /** @var Collection| ContaBancariaSync[] $meses */
    $meses = $this->ContaBancariaSync()
                  ->where( 'data', '>', $dataInicioAlt->format( 'Y-m-d' ) )
                  ->orderBy( 'data', 'asc' )
                  ->get();
    if ( !$meses ) {
      return;
    }
    // novo valor deste mes inteiro
    $valorAlt      = $this->getValorTotalPeriodo( $dataInicioAlt, $dataFimAlt );
    $novoProxValor = $ContaMesAlt->valor + $valorAlt;

    $cont = count( $meses );
    foreach ( $meses as $mes ) {
      // atualiza o inicial do prox mes
      $mes->valor = $novoProxValor;
      $mes->save();

      // calcular o valor total do mes somando ao inicial para salvar no prox
      if ( $cont > 1 ) {
        $dataInicio = $mes->data->copy()->startOfMonth();
        $dataFim    = $mes->data->copy()->endOfMonth();

        $valorDoMes    = $this->getValorTotalPeriodo( $dataInicio, $dataFim );
        $novoProxValor = $mes->valor + $valorDoMes;
      }
      $cont--;
    }
  }
}
