<?php

namespace App\Models\Financeiro;

use App\ListagemPadronizada;
use Illuminate\Database\Eloquent\Model;

/**
 * App\Models\Financeiro\Investimentos
 *
 * @property int $id
 * @property string $nome
 * @property string $data
 * @property string $rendimento
 * @property int $valor_bruto
 * @property int $valor_ir
 * @property int $valor_liquido
 * @property int $conta_id
 * @property \Carbon\Carbon $created_at
 * @property \Carbon\Carbon $updated_at
 * @property-read \App\Models\Financeiro\ContaBancaria $ContaBancaria
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereContaId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereCreatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereData( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereNome( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereRendimento( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereUpdatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereValorBruto( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereValorIr( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Investimentos whereValorLiquido( $value )
 * @mixin \Eloquent
 */
class Investimentos extends Model {
  /**
   * A tabela associada com este Model.
   *
   * @var string
   */
  protected $table = 'financeiro_investimentos';

  /**
   * Os atributos que são atribuíveis em massa.
   *
   * @var array
   */
  protected $fillable = [
    'nome',
    'data',
    'conta_id',

    'rendimento',
    'valor_bruto',
    'valor_ir',
    'valor_liquido',
  ];

  /**
   * Retorna a ContaBancaria q este Salario pertence.
   *
   * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
   */
  public function ContaBancaria() {
    return $this->belongsTo( ContaBancaria::class, 'conta_id' );
  }

  /**
   * Retorna a ContaBancaria q este Salario pertence.
   *
   * @return string
   */
  public function ContaBancariaNome() {
    return $this->ContaBancaria ? $this->ContaBancaria->nome : 'SEM CONTA ASSOCIADA';
  }

  /**
   * Retorna os Salarios entre as datas de uma conta.
   *
   * @param string $inicio
   * @param string $fim
   * @param int $conta_id
   * @return \Illuminate\Support\Collection|static[] | Investimentos[]
   */
  public static function Periodo( $inicio, $fim, $conta_id ) {
    return Investimentos
      ::whereBetween( 'data', [ $inicio, $fim ] )
      ->whereContaId( $conta_id )
      ->orderBy( 'data' )
      ->get();
  }

  /**
   * Armazena a data antiga antes do update.
   *
   * @var string
   */
  private $dtAntiga = null;

  /**
   * Armazena a data antiga para o reSync da Conta Bancaria.
   */
  public function reSyncUpdateBegin() {
    $this->dtAntiga = $this->data;
  }

  /**
   * Realiza o reSync da Conta Bancaria quando update.
   */
  public function reSyncUpdateEnd() {
    if ( $this->dtAntiga === null ) {
      dd( 'reSync_update_End sem reSync_update_Begin' );
    }

    if ( $this->data < $this->dtAntiga ) {
      $this->ContaBancaria->reSyncConta( $this->data );
    } else {
      $this->ContaBancaria->reSyncConta( $this->dtAntiga );
    }
  }

  /**
   * Realiza o reSync da Conta Bancaria quando criado este Evento.
   */
  public function reSyncStore() {
    $this->ContaBancaria->reSyncConta( $this->data );
  }

  /**
   * Realiza o reSync da Conta Bancaria quando deletado este Evento.
   */
  public function reSyncDestroy() {
    $this->ContaBancaria->reSyncConta( $this->data );
  }

  use ListagemPadronizada;

  public $sortable = [ 'nome', 'rendimento', 'data', 'conta_id', 'valor_liquido' ];

  public static function IndexSearchMap() : array {
    return [
      'search_text'     => [
        'fields' => [ 'nome', 'rendimento' ],
        'label'  => 'Pesquise pelo Nome ou Rendimento'
      ],
      'search_advanced' => [
        'conta_id' => [
          'type'    => self::$LB_TYPE_SELECT,
          'label'   => 'Conta Bancária:',
          'options' => ContaBancaria::enum_contas()
        ],
        'data'     => [
          'type'         => self::$LB_TYPE_DATE_RANGE,
          'label_after'  => 'Data após:',
          'label_before' => 'Data antes de:',
        ]
      ],
    ];
  }
}
