<?php

namespace App\Models\Financeiro;

use App\Models\Comercial\Jobs;
use Illuminate\Database\Eloquent\Model;

/**
 * App\Models\Financeiro\JobParcela
 *
 * @property int $id
 * @property string $nome
 * @property \Carbon\Carbon $data
 * @property bool $pago
 * @property int $valor_bruto
 * @property int $valor_imposto
 * @property int $valor_liquido
 * @property string $nota_fiscal
 * @property string $nota_fiscal_data_emissao
 * @property int $job_id
 * @property int $conta_id
 * @property \Carbon\Carbon $created_at
 * @property \Carbon\Carbon $updated_at
 * @property-read \App\Models\Financeiro\ContaBancaria $ContaBancaria
 * @property-read \App\Models\Comercial\Jobs $Job
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereContaId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereCreatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereData( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereJobId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereNome( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereNotaFiscal( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereNotaFiscalDataEmissao( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela wherePago( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereUpdatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereValorBruto( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereValorImposto( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\JobParcela whereValorLiquido( $value )
 * @mixin \Eloquent
 */
class JobParcela extends Model {
  /**
   * A tabela associada com este Model.
   *
   * @var string
   */
  protected $table = 'financeiro_jobs_parcelas';

  /**
   * Os atributos que são atribuíveis em massa.
   *
   * @var array
   */
  protected $fillable = [
    'nome',
    'data',
    'pago',
    'valor_bruto',
    'valor_imposto',
    'valor_liquido',
    'nota_fiscal',
    'nota_fiscal_data_emissao',
    'job_id',
    'conta_id',
  ];

  /**
   * Propriedades q sao datas.
   *
   * @return array
   */
  public function getDates() {
    return [ 'created_at', 'updated_at', 'data' ];
  }

  /**
   * Retorna o Job q esta Parcela pertence.
   *
   * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
   */
  public function Job() {
    return $this->belongsTo( Jobs::class, 'job_id' );
  }

  /**
   * Retorna a ContaBancaria q esta Parcela pertence.
   *
   * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
   */
  public function ContaBancaria() {
    return $this->belongsTo( ContaBancaria::class, 'conta_id' );
  }

  /**
   * Retorna a ContaBancaria q esta Parcela pertence.
   *
   * @return string
   */
  public function ContaBancariaNome() {
    return $this->ContaBancaria ? $this->ContaBancaria->nome : 'SEM CONTA ASSOCIADA';
  }

  /**
   * Retorna os JobParcela entre as datas de uma conta.
   *
   * @param string $inicio
   * @param string $fim
   * @param int $conta_id
   * @return \Illuminate\Database\Eloquent\Collection|\Illuminate\Support\Collection|static[] | JobParcela[]
   */
  public static function Periodo( $inicio, $fim, $conta_id ) {
    return JobParcela
      ::whereBetween( 'data', [ $inicio, $fim ] )
      ->whereContaId( $conta_id )
      ->orderBy( 'data' )
      ->get();
  }

  /**
   * Armazena a data antiga antes do update.
   *
   * @var string
   */
  private $dtAntiga = null;

  /**
   * Armazena se estava pago antes do update.
   *
   * @var bool
   */
  private $pagoAntiga = null;

  /**
   * Armazena qual a Conta Bancaria antes do update.
   *
   * @var int
   */
  private $conta_idAntiga = null;

  /**
   * Armazena a data antiga para o reSync da Conta Bancaria.
   */
  public function reSyncUpdateBegin() {
    $this->dtAntiga       = $this->data;
    $this->pagoAntiga     = $this->pago;
    $this->conta_idAntiga = $this->conta_id;
  }

  /**
   * Realiza o reSync da Conta Bancaria quando update.
   */
  public function reSyncUpdateEnd() {
    if ( $this->dtAntiga === null || $this->pagoAntiga === null ) {
      dd( 'reSync_update_End sem reSync_update_Begin' );
    }

    if ( $this->conta_idAntiga == $this->conta_id ) { // nao mudou a conta
      if ( $this->ContaBancaria ) { // nao eh null
        $this->ContaBancaria->reSyncConta( $this->data ); // sync
      }
    } else { // diferentes, sync os dois se possivel
      if ( $this->ContaBancaria ) {
        $this->ContaBancaria->reSyncConta( $this->data );
      }
      if ( $this->conta_idAntiga ) {
        $conta = ContaBancaria::find( $this->conta_idAntiga );
        if ( $conta ) {
          $conta->reSyncConta( $this->data );
        }
      }
    }
  }

  /**
   * Realiza o reSync da Conta Bancaria quando criado este Evento.
   */
  public function reSyncStore() {
    if ( $this->pago == true ) {
      if ( $this->ContaBancaria ) {
        $this->ContaBancaria->reSyncConta( $this->data );
      }
    }
  }

  /**
   * Realiza o reSync da Conta Bancaria quando deletado este Evento.
   */
  public function reSyncDestroy() {
    if ( $this->pago == true ) {
      if ( $this->ContaBancaria ) {
        $this->pago = false;
        $this->save();
        $this->ContaBancaria->reSyncConta( $this->data );
      }
    }
  }
}
