<?php

namespace App\Models\Financeiro;

use App\ListagemPadronizada;
use App\Models\Cadastros\Funcionarios;
use Illuminate\Database\Eloquent\Model;

/**
 * App\Models\Financeiro\Transferencia
 *
 * @property int $id
 * @property string $nome
 * @property string $data
 * @property int $valor
 * @property int $valor_taxa
 * @property int $conta_saida_id
 * @property int $conta_entrada_id
 * @property \Carbon\Carbon $created_at
 * @property \Carbon\Carbon $updated_at
 * @property-read \App\Models\Financeiro\ContaBancaria $ContaBancariaEntrada
 * @property-read \App\Models\Financeiro\ContaBancaria $ContaBancariaSaida
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereContaEntradaId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereContaSaidaId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereCreatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereData( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereId( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereNome( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereUpdatedAt( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereValor( $value )
 * @method static \Illuminate\Database\Query\Builder|\App\Models\Financeiro\Transferencia whereValorTaxa( $value )
 * @mixin \Eloquent
 */
class Transferencia extends Model {
  /**
   * A tabela associada com este Model.
   *
   * @var string
   */
  protected $table = 'financeiro_transferencias';

  /**
   * Os atributos que são atribuíveis em massa.
   *
   * @var array
   */
  protected $fillable = [
    'nome',
    'data',
    'valor',
    'valor_taxa',
    'conta_saida_id',
    'conta_entrada_id',
  ];

  /**
   * Retorna a ContaBancaria q esta Parcela pertence.
   *
   * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
   */
  public function ContaBancariaSaida() {
    return $this->belongsTo( ContaBancaria::class, 'conta_saida_id' );
  }

  /**
   * Retorna o nome da ContaBancariaSaida desta Transferencia.
   *
   * @return string
   */
  public function ContaBancariaSaidaNome() {
    return $this->ContaBancariaSaida ? $this->ContaBancariaSaida->nome : 'SEM CONTA ASSOCIADA';
  }


  /**
   * Retorna a ContaBancaria q esta Parcela pertence.
   *
   * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
   */
  public function ContaBancariaEntrada() {
    return $this->belongsTo( ContaBancaria::class, 'conta_entrada_id' );
  }

  /**
   * Retorna o nome da ContaBancariaSaida desta Transferencia.
   *
   * @return string
   */
  public function ContaBancariaEntradaNome() {
    return $this->ContaBancariaEntrada ? $this->ContaBancariaEntrada->nome : 'SEM CONTA ASSOCIADA';
  }

  /**
   * Retorna as Transferências entre as datas de uma conta.
   *
   * @param string $inicio
   * @param string $fim
   * @param int $conta_id
   * @return \Illuminate\Database\Eloquent\Collection|\Illuminate\Support\Collection|static[] | Transferencia[]
   */
  public static function PeriodoEntrada( $inicio, $fim, $conta_id ) {
    return Transferencia
      ::whereBetween( 'data', [ $inicio, $fim ] )
      ->whereContaEntradaId( $conta_id )
      ->orderBy( 'data' )
      ->get();
  }

  /**
   * Retorna as Transferências entre as datas de uma conta.
   *
   * @param string $inicio
   * @param string $fim
   * @param int $conta_id
   * @return \Illuminate\Database\Eloquent\Collection|\Illuminate\Support\Collection|static[] | Transferencia[]
   */
  public static function PeriodoSaida( $inicio, $fim, $conta_id ) {
    return Transferencia
      ::whereBetween( 'data', [ $inicio, $fim ] )
      ->whereContaSaidaId( $conta_id )
      ->orderBy( 'data' )
      ->get();
  }

  /**
   * Armazena a data antiga antes do update.
   *
   * @var string
   */
  private $dtAntiga = null;

  /**
   * Armazena a data antiga para o reSync da Conta Bancaria.
   */
  public function reSyncUpdateBegin() {
    $this->dtAntiga = $this->data;
  }

  /**
   * Realiza o reSync da Conta Bancaria quando update.
   */
  public function reSyncUpdateEnd() {
    if ( $this->dtAntiga === null ) {
      dd( 'reSync_update_End sem reSync_update_Begin' );
    }
    if ( $this->data < $this->dtAntiga ) {
      $this->ContaBancariaEntrada->reSyncConta( $this->data );
      $this->ContaBancariaSaida->reSyncConta( $this->data );
    } else {
      $this->ContaBancariaEntrada->reSyncConta( $this->dtAntiga );
      $this->ContaBancariaSaida->reSyncConta( $this->dtAntiga );
    }
  }

  /**
   * Realiza o reSync da Conta Bancaria quando criado este Evento.
   */
  public function reSyncStore() {
    $this->ContaBancariaEntrada->reSyncConta( $this->data );
    $this->ContaBancariaSaida->reSyncConta( $this->data );
  }

  /**
   * Realiza o reSync da Conta Bancaria quando deletado este Evento.
   */
  public function reSyncDestroy() {
    $this->ContaBancariaEntrada->reSyncConta( $this->data );
    $this->ContaBancariaSaida->reSyncConta( $this->data );
  }

  use ListagemPadronizada;

  public $sortable = [ 'nome', 'data', 'conta_saida_id', 'conta_entrada_id', 'valor' ];

  public static function IndexSearchMap() : array {
    return [
      'search_text'     => [
        'fields' => [ 'nome' ],
        'label'  => 'Pesquise pelo Nome'
      ],
      'search_advanced' => [
        'conta_saida_id'   => [
          'type'    => self::$LB_TYPE_SELECT,
          'label'   => 'Conta Origem:',
          'options' => ContaBancaria::enum_contas()
        ],
        'conta_entrada_id' => [
          'type'    => self::$LB_TYPE_SELECT,
          'label'   => 'Conta Destino:',
          'options' => ContaBancaria::enum_contas()
        ],
        'data'             => [
          'type'         => self::$LB_TYPE_DATE_RANGE,
          'label_after'  => 'Data após:',
          'label_before' => 'Data antes de:',
        ]
      ],
    ];
  }


}
